"""
Unit tests for CLI module
"""

import sys
from io import StringIO
from unittest.mock import patch

import pytest

from mono_kickstart import __version__
from mono_kickstart.cli import create_parser, main


def test_version_command():
    """Test --version command displays correct version"""
    with patch('sys.argv', ['mk', '--version']):
        with patch('sys.stdout', new=StringIO()) as fake_out:
            with pytest.raises(SystemExit) as exc_info:
                parser = create_parser()
                parser.parse_args()
            assert exc_info.value.code == 0
            assert __version__ in fake_out.getvalue()


def test_help_command():
    """Test --help command displays help information"""
    with patch('sys.argv', ['mk', '--help']):
        with patch('sys.stdout', new=StringIO()) as fake_out:
            with pytest.raises(SystemExit) as exc_info:
                parser = create_parser()
                parser.parse_args()
            assert exc_info.value.code == 0
            output = fake_out.getvalue()
            assert "Monorepo 项目模板脚手架 CLI 工具" in output
            assert "init" in output
            assert "upgrade" in output


def test_init_help():
    """Test init --help command"""
    with patch('sys.argv', ['mk', 'init', '--help']):
        with patch('sys.stdout', new=StringIO()) as fake_out:
            with pytest.raises(SystemExit) as exc_info:
                parser = create_parser()
                parser.parse_args()
            assert exc_info.value.code == 0
            output = fake_out.getvalue()
            assert "初始化 Monorepo 项目和开发环境" in output
            assert "--config" in output
            assert "--save-config" in output
            assert "--interactive" in output
            assert "--force" in output
            assert "--dry-run" in output


def test_upgrade_help():
    """Test upgrade --help command"""
    with patch('sys.argv', ['mk', 'upgrade', '--help']):
        with patch('sys.stdout', new=StringIO()) as fake_out:
            with pytest.raises(SystemExit) as exc_info:
                parser = create_parser()
                parser.parse_args()
            assert exc_info.value.code == 0
            output = fake_out.getvalue()
            assert "升级已安装的开发工具" in output
            assert "--all" in output
            assert "--dry-run" in output


# ============================================================================
# Complete Flow Tests
# ============================================================================


def test_init_complete_flow_success_mocked(tmp_path, monkeypatch):
    """Test init command complete flow with successful installation (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_init.return_value = {
                'nvm': InstallReport('nvm', InstallResult.SUCCESS, 'Installed successfully', '0.40.4'),
                'node': InstallReport('node', InstallResult.SUCCESS, 'Installed successfully', '20.0.0'),
            }
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'init', '--dry-run']):
                exit_code = main()
                
                assert exit_code == 0
                assert mock_orch.run_init.called
                assert mock_orch.print_summary.called


def test_init_complete_flow_partial_failure_mocked(tmp_path, monkeypatch):
    """Test init command with partial failures (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_init.return_value = {
                'nvm': InstallReport('nvm', InstallResult.SUCCESS, 'Installed successfully', '0.40.4'),
                'node': InstallReport('node', InstallResult.FAILED, 'Installation failed', error='Network error'),
            }
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'init', '--dry-run']):
                exit_code = main()
                assert exit_code == 0  # Partial failure should return 0


def test_init_complete_flow_all_failures_mocked(tmp_path, monkeypatch):
    """Test init command when all installations fail (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_init.return_value = {
                'nvm': InstallReport('nvm', InstallResult.FAILED, 'Installation failed', error='Network error'),
                'node': InstallReport('node', InstallResult.FAILED, 'Installation failed', error='Network error'),
            }
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'init', '--dry-run']):
                exit_code = main()
                assert exit_code == 3  # All failures should return 3


def test_upgrade_complete_flow_success_mocked(tmp_path, monkeypatch):
    """Test upgrade command complete flow with successful upgrades (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.tool_detector import ToolStatus
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.tool_detector.ToolDetector') as mock_tool_detector_class:
            mock_tool_detector = mock_tool_detector_class.return_value
            mock_tool_detector.detect_all_tools.return_value = {
                'nvm': ToolStatus('nvm', True, '0.40.3', '/usr/local/bin/nvm'),
                'node': ToolStatus('node', True, '18.0.0', '/usr/local/bin/node'),
            }
            
            with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
                mock_orch = mock_orch_class.return_value
                mock_orch.run_upgrade.return_value = {
                    'nvm': InstallReport('nvm', InstallResult.SUCCESS, 'Upgraded successfully', '0.40.4'),
                    'node': InstallReport('node', InstallResult.SUCCESS, 'Upgraded successfully', '20.0.0'),
                }
                mock_orch.print_summary.return_value = None
                
                with patch('sys.argv', ['mk', 'upgrade', '--all', '--dry-run']):
                    exit_code = main()
                    assert exit_code == 0
                    assert mock_orch.run_upgrade.called


def test_upgrade_single_tool_mocked(tmp_path, monkeypatch):
    """Test upgrade command for a single tool (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_upgrade.return_value = {
                'node': InstallReport('node', InstallResult.SUCCESS, 'Upgraded successfully', '20.0.0'),
            }
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'upgrade', 'node', '--dry-run']):
                exit_code = main()
                assert exit_code == 0


def test_install_complete_flow_success_mocked(tmp_path, monkeypatch):
    """Test install command complete flow with successful installation (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.install_tool.return_value = InstallReport(
                'bun', InstallResult.SUCCESS, 'Installed successfully', '1.0.0'
            )
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'install', 'bun', '--dry-run']):
                exit_code = main()
                assert exit_code == 0
                assert mock_orch.install_tool.called


def test_install_all_tools_mocked(tmp_path, monkeypatch):
    """Test install command with --all flag (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.install_all_tools.return_value = {
                'nvm': InstallReport('nvm', InstallResult.SUCCESS, 'Installed successfully', '0.40.4'),
                'node': InstallReport('node', InstallResult.SUCCESS, 'Installed successfully', '20.0.0'),
            }
            mock_orch.print_summary.return_value = None
            
            with patch('sys.argv', ['mk', 'install', '--all', '--dry-run']):
                exit_code = main()
                assert exit_code == 0
                assert mock_orch.install_all_tools.called


def test_install_without_tool_or_all_flag_mocked(tmp_path, monkeypatch):
    """Test install command fails when neither tool nor --all is specified (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('sys.argv', ['mk', 'install', '--dry-run']):
            exit_code = main()
            assert exit_code == 1


# ============================================================================
# Error Handling Tests
# ============================================================================


def test_init_unsupported_platform_mocked():
    """Test init command fails on unsupported platform (mocked)"""
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.UNSUPPORTED,
        arch=Arch.UNSUPPORTED,
        shell=Shell.BASH,
        shell_config_file=""
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = False
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('sys.argv', ['mk', 'init']):
            exit_code = main()
            assert exit_code == 1


def test_upgrade_unsupported_platform_mocked():
    """Test upgrade command fails on unsupported platform (mocked)"""
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.UNSUPPORTED,
        arch=Arch.UNSUPPORTED,
        shell=Shell.BASH,
        shell_config_file=""
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = False
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('sys.argv', ['mk', 'upgrade', '--all']):
            exit_code = main()
            assert exit_code == 1


def test_install_unsupported_platform_mocked():
    """Test install command fails on unsupported platform (mocked)"""
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.UNSUPPORTED,
        arch=Arch.UNSUPPORTED,
        shell=Shell.BASH,
        shell_config_file=""
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = False
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('sys.argv', ['mk', 'install', '--all']):
            exit_code = main()
            assert exit_code == 1


def test_init_config_file_not_found_mocked(tmp_path, monkeypatch):
    """Test init command handles missing config file gracefully (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.config.ConfigManager') as mock_config_class:
            mock_config = mock_config_class.return_value
            mock_config.load_with_priority.side_effect = FileNotFoundError("Config file not found")
            
            with patch('sys.argv', ['mk', 'init', '--config', 'nonexistent.yaml']):
                exit_code = main()
                assert exit_code == 2


def test_init_config_validation_error_mocked(tmp_path, monkeypatch):
    """Test init command handles config validation errors (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.config import Config
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.config.ConfigManager') as mock_config_class:
            mock_config = mock_config_class.return_value
            mock_config.load_with_priority.return_value = Config()
            mock_config.validate.return_value = ["Invalid tool name: invalid-tool"]
            
            with patch('sys.argv', ['mk', 'init']):
                exit_code = main()
                assert exit_code == 2


def test_init_keyboard_interrupt_mocked(tmp_path, monkeypatch):
    """Test init command handles keyboard interrupt gracefully (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_init.side_effect = KeyboardInterrupt()
            
            with patch('sys.argv', ['mk', 'init', '--dry-run']):
                exit_code = main()
                assert exit_code == 130


def test_upgrade_keyboard_interrupt_mocked(tmp_path, monkeypatch):
    """Test upgrade command handles keyboard interrupt gracefully (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_upgrade.side_effect = KeyboardInterrupt()
            
            with patch('sys.argv', ['mk', 'upgrade', '--all', '--dry-run']):
                exit_code = main()
                assert exit_code == 130


def test_install_keyboard_interrupt_mocked(tmp_path, monkeypatch):
    """Test install command handles keyboard interrupt gracefully (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.install_all_tools.side_effect = KeyboardInterrupt()
            
            with patch('sys.argv', ['mk', 'install', '--all', '--dry-run']):
                exit_code = main()
                assert exit_code == 130


def test_init_unexpected_exception_mocked(tmp_path, monkeypatch):
    """Test init command handles unexpected exceptions (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
            mock_orch = mock_orch_class.return_value
            mock_orch.run_init.side_effect = RuntimeError("Unexpected error")
            
            with patch('sys.argv', ['mk', 'init', '--dry-run']):
                exit_code = main()
                assert exit_code == 1


def test_upgrade_no_installed_tools_mocked(tmp_path, monkeypatch):
    """Test upgrade command when no tools are installed (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.tool_detector import ToolStatus
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.tool_detector.ToolDetector') as mock_tool_detector_class:
            mock_tool_detector = mock_tool_detector_class.return_value
            mock_tool_detector.detect_all_tools.return_value = {
                'nvm': ToolStatus('nvm', False, None, None),
                'node': ToolStatus('node', False, None, None),
            }
            
            with patch('sys.argv', ['mk', 'upgrade', '--all', '--dry-run']):
                exit_code = main()
                assert exit_code == 0


def test_init_with_save_config_mocked(tmp_path, monkeypatch):
    """Test init command saves config when --save-config is used (mocked)"""
    monkeypatch.chdir(tmp_path)
    
    from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell
    from mono_kickstart.config import Config
    from mono_kickstart.installer_base import InstallReport, InstallResult
    
    mock_platform = PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file=str(tmp_path / ".bashrc")
    )
    
    with patch('mono_kickstart.platform_detector.PlatformDetector') as mock_detector_class:
        mock_detector = mock_detector_class.return_value
        mock_detector.is_supported.return_value = True
        mock_detector.detect_all.return_value = mock_platform
        
        with patch('mono_kickstart.config.ConfigManager') as mock_config_class:
            mock_config = mock_config_class.return_value
            mock_config.load_with_priority.return_value = Config()
            mock_config.validate.return_value = []
            
            with patch('mono_kickstart.orchestrator.InstallOrchestrator') as mock_orch_class:
                mock_orch = mock_orch_class.return_value
                mock_orch.run_init.return_value = {
                    'nvm': InstallReport('nvm', InstallResult.SUCCESS, 'Installed successfully', '0.40.4'),
                }
                mock_orch.print_summary.return_value = None
                
                with patch('sys.argv', ['mk', 'init', '--save-config', '--dry-run']):
                    exit_code = main()
                    
                    assert exit_code == 0
                    # Verify save_to_file was called
                    assert mock_config.save_to_file.called



"""
项目创建器的单元测试
"""

import json
import subprocess
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from mono_kickstart.project_creator import ProjectCreator


class TestProjectCreator:
    """测试 ProjectCreator 类"""

    def test_init(self, tmp_path):
        """测试初始化"""
        project_name = "test-project"
        creator = ProjectCreator(project_name, tmp_path)

        assert creator.project_name == project_name
        assert creator.project_path == tmp_path

    def test_create_directory_structure(self, tmp_path):
        """测试创建目录结构"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        creator.create_directory_structure()

        # 验证主目录存在
        assert project_path.exists()
        assert project_path.is_dir()

        # 验证子目录存在
        assert (project_path / "apps").exists()
        assert (project_path / "packages").exists()
        assert (project_path / "shared").exists()

        # 验证 .gitkeep 文件存在
        assert (project_path / "apps" / ".gitkeep").exists()
        assert (project_path / "packages" / ".gitkeep").exists()
        assert (project_path / "shared" / ".gitkeep").exists()

    def test_create_directory_structure_idempotent(self, tmp_path):
        """测试创建目录结构的幂等性"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        # 第一次创建
        creator.create_directory_structure()
        # 第二次创建（应该不报错）
        creator.create_directory_structure()

        # 验证目录仍然存在
        assert project_path.exists()
        assert (project_path / "apps").exists()

    def test_create_workspace_config(self, tmp_path):
        """测试创建 workspace 配置文件"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        creator.create_workspace_config()

        # 验证 package.json 存在
        package_json_path = project_path / "package.json"
        assert package_json_path.exists()

        # 验证 package.json 内容
        with open(package_json_path, "r", encoding="utf-8") as f:
            package_json = json.load(f)

        assert package_json["name"] == project_name
        assert package_json["version"] == "0.1.0"
        assert package_json["private"] is True
        assert package_json["workspaces"] == ["apps/*", "packages/*"]
        assert "scripts" in package_json
        assert "dev" in package_json["scripts"]
        assert "build" in package_json["scripts"]
        assert "test" in package_json["scripts"]

        # 验证 pnpm-workspace.yaml 存在
        pnpm_workspace_path = project_path / "pnpm-workspace.yaml"
        assert pnpm_workspace_path.exists()

        # 验证 pnpm-workspace.yaml 内容
        with open(pnpm_workspace_path, "r", encoding="utf-8") as f:
            content = f.read()

        assert "packages:" in content
        assert "'apps/*'" in content
        assert "'packages/*'" in content

    def test_create_gitignore(self, tmp_path):
        """测试创建 .gitignore 文件"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        creator.create_gitignore()

        # 验证 .gitignore 存在
        gitignore_path = project_path / ".gitignore"
        assert gitignore_path.exists()

        # 验证 .gitignore 内容
        with open(gitignore_path, "r", encoding="utf-8") as f:
            content = f.read()

        # 检查关键条目
        assert "node_modules/" in content
        assert "dist/" in content
        assert ".env" in content
        assert "__pycache__/" in content
        assert ".DS_Store" in content

    def test_create_readme(self, tmp_path):
        """测试创建 README.md 文件"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        creator.create_readme()

        # 验证 README.md 存在
        readme_path = project_path / "README.md"
        assert readme_path.exists()

        # 验证 README.md 内容
        with open(readme_path, "r", encoding="utf-8") as f:
            content = f.read()

        assert project_name in content
        assert "项目结构" in content
        assert "开始使用" in content
        assert "pnpm install" in content

    def test_init_git_repo_success(self, tmp_path):
        """测试成功初始化 Git 仓库"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        # 创建一些文件以便提交
        (project_path / "test.txt").write_text("test")

        result = creator.init_git_repo()

        # 验证返回值
        assert result is True

        # 验证 .git 目录存在
        assert (project_path / ".git").exists()

    def test_init_git_repo_already_exists(self, tmp_path):
        """测试 Git 仓库已存在的情况"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        (project_path / ".git").mkdir()
        creator = ProjectCreator(project_name, project_path)

        result = creator.init_git_repo()

        # 验证返回值（已存在也返回 True）
        assert result is True

    def test_init_git_repo_git_not_installed(self, tmp_path):
        """测试 Git 未安装的情况"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        with patch("subprocess.run", side_effect=FileNotFoundError):
            result = creator.init_git_repo()

        # 验证返回值（失败返回 False）
        assert result is False

    def test_init_git_repo_timeout(self, tmp_path):
        """测试 Git 初始化超时的情况"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        with patch("subprocess.run", side_effect=subprocess.TimeoutExpired("git", 10)):
            result = creator.init_git_repo()

        # 验证返回值（超时返回 False）
        assert result is False

    def test_check_existing_directory_not_exists(self, tmp_path):
        """测试检查不存在的目录"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        exists, items = creator.check_existing_directory()

        assert exists is False
        assert items == []

    def test_check_existing_directory_empty(self, tmp_path):
        """测试检查空目录"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        exists, items = creator.check_existing_directory()

        assert exists is True
        assert items == []

    def test_check_existing_directory_with_files(self, tmp_path):
        """测试检查包含文件的目录"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        (project_path / "file1.txt").write_text("test")
        (project_path / "file2.txt").write_text("test")
        (project_path / ".hidden").write_text("test")  # 隐藏文件应该被忽略
        creator = ProjectCreator(project_name, project_path)

        exists, items = creator.check_existing_directory()

        assert exists is True
        assert len(items) == 2
        assert "file1.txt" in items
        assert "file2.txt" in items
        assert ".hidden" not in items

    def test_create_project_success(self, tmp_path):
        """测试成功创建项目"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        success, error = creator.create_project()

        # 验证返回值
        assert success is True
        assert error is None

        # 验证目录结构
        assert project_path.exists()
        assert (project_path / "apps").exists()
        assert (project_path / "packages").exists()
        assert (project_path / "shared").exists()

        # 验证配置文件
        assert (project_path / "package.json").exists()
        assert (project_path / "pnpm-workspace.yaml").exists()

        # 验证其他文件
        assert (project_path / ".gitignore").exists()
        assert (project_path / "README.md").exists()

    def test_create_project_existing_directory_without_force(self, tmp_path):
        """测试在已存在目录上创建项目（不使用 force）"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        (project_path / "existing.txt").write_text("test")
        creator = ProjectCreator(project_name, project_path)

        success, error = creator.create_project(force=False)

        # 验证返回值
        assert success is False
        assert error is not None
        assert "已存在" in error
        assert "--force" in error

    def test_create_project_existing_directory_with_force(self, tmp_path):
        """测试在已存在目录上创建项目（使用 force）"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        (project_path / "existing.txt").write_text("test")
        creator = ProjectCreator(project_name, project_path)

        success, error = creator.create_project(force=True)

        # 验证返回值
        assert success is True
        assert error is None

        # 验证项目文件已创建
        assert (project_path / "package.json").exists()

    def test_create_project_empty_directory(self, tmp_path):
        """测试在空目录上创建项目"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        project_path.mkdir()
        creator = ProjectCreator(project_name, project_path)

        success, error = creator.create_project()

        # 验证返回值（空目录应该成功）
        assert success is True
        assert error is None

    def test_create_project_permission_error(self, tmp_path):
        """测试权限错误"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        with patch.object(Path, "mkdir", side_effect=PermissionError("Permission denied")):
            success, error = creator.create_project()

        # 验证返回值
        assert success is False
        assert error is not None
        assert "权限错误" in error

    def test_create_project_os_error(self, tmp_path):
        """测试文件系统错误"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        with patch.object(Path, "mkdir", side_effect=OSError("Disk full")):
            success, error = creator.create_project()

        # 验证返回值
        assert success is False
        assert error is not None
        assert "文件系统错误" in error

    def test_create_project_git_init_failure_not_fatal(self, tmp_path):
        """测试 Git 初始化失败不影响项目创建"""
        project_name = "test-project"
        project_path = tmp_path / project_name
        creator = ProjectCreator(project_name, project_path)

        with patch.object(creator, "init_git_repo", return_value=False):
            success, error = creator.create_project()

        # 验证项目仍然创建成功
        assert success is True
        assert error is None
        assert (project_path / "package.json").exists()



"""Conda 安装器模块

该模块实现 Conda (Miniconda) 的安装、升级和验证逻辑。
"""

import tempfile
from pathlib import Path
from typing import Optional

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo, OS, Arch


class CondaInstaller(ToolInstaller):
    """Conda 安装器
    
    负责安装、升级和验证 Conda (Miniconda)。
    Conda 通过下载并执行平台特定的 Miniconda 安装脚本进行安装。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        install_dir: Conda 安装目录（默认 ~/miniconda3）
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 Conda 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # Conda 安装目录
        self.install_dir = Path.home() / "miniconda3"
    
    def get_install_url(self) -> str:
        """根据平台选择正确的 Miniconda 安装包 URL
        
        根据操作系统和架构选择对应的安装包：
        - Linux x86_64: Miniconda3-latest-Linux-x86_64.sh
        - macOS ARM64: Miniconda3-latest-MacOSX-arm64.sh
        - macOS x86_64: Miniconda3-latest-MacOSX-x86_64.sh
        
        Returns:
            str: 安装包 URL
            
        Raises:
            ValueError: 如果平台不支持
        """
        base_url = "https://mirrors.sustech.edu.cn/anaconda/miniconda"
        
        # 根据平台选择安装包
        if self.platform_info.os == OS.LINUX and self.platform_info.arch == Arch.X86_64:
            installer_name = "Miniconda3-latest-Linux-x86_64.sh"
        elif self.platform_info.os == OS.MACOS and self.platform_info.arch == Arch.ARM64:
            installer_name = "Miniconda3-latest-MacOSX-arm64.sh"
        elif self.platform_info.os == OS.MACOS and self.platform_info.arch == Arch.X86_64:
            installer_name = "Miniconda3-latest-MacOSX-x86_64.sh"
        else:
            raise ValueError(
                f"不支持的平台: {self.platform_info.os.value} {self.platform_info.arch.value}"
            )
        
        return f"{base_url}/{installer_name}"
    
    def verify(self) -> bool:
        """验证 Conda 是否已正确安装
        
        检查 conda 命令是否可用，并尝试执行 conda --version 命令。
        
        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        # 检查安装目录是否存在
        if not self.install_dir.exists():
            return False
        
        # 检查 conda 可执行文件是否存在
        conda_bin = self.install_dir / "bin" / "conda"
        if not conda_bin.exists():
            return False
        
        # 尝试执行 conda --version 命令
        returncode, stdout, stderr = self.run_command(
            f"{conda_bin} --version",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 Conda 版本
        
        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        conda_bin = self.install_dir / "bin" / "conda"
        
        if not conda_bin.exists():
            return None
        
        returncode, stdout, stderr = self.run_command(
            f"{conda_bin} --version",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        if returncode == 0:
            # conda --version 输出格式: "conda 23.1.0"
            # 提取版本号
            parts = stdout.strip().split()
            if len(parts) >= 2:
                return parts[1]
        
        return None
    
    def _download_installer(self, url: str, dest: str) -> bool:
        """使用 curl 下载安装脚本

        使用外部 curl 进程下载，避免 Python urllib 在 conda 环境下
        因 SSL 库冲突导致 segfault。

        Args:
            url: 下载 URL
            dest: 目标文件路径

        Returns:
            bool: 下载成功返回 True，否则返回 False
        """
        returncode, stdout, stderr = self.run_command(
            f"curl -fsSL -o {dest} {url}",
            shell=True,
            timeout=300,
            max_retries=3
        )
        return returncode == 0 and Path(dest).exists() and Path(dest).stat().st_size > 0

    def install(self) -> InstallReport:
        """安装 Conda
        
        执行以下步骤：
        1. 检查 Conda 是否已安装
        2. 根据平台选择正确的安装包 URL
        3. 下载 Miniconda 安装脚本
        4. 执行安装脚本（批量模式，不修改 shell 配置）
        5. 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="conda",
                result=InstallResult.SKIPPED,
                message="Conda 已安装，跳过安装",
                version=version
            )
        
        try:
            # 获取安装包 URL
            try:
                install_url = self.get_install_url()
            except ValueError as e:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="不支持的平台",
                    error=str(e)
                )
            
            # 下载安装脚本到临时文件
            with tempfile.NamedTemporaryFile(
                mode='w',
                suffix='.sh',
                delete=False
            ) as temp_file:
                temp_script_path = temp_file.name
            
            # 下载安装脚本（使用 curl 避免 Python SSL 库冲突）
            download_success = self._download_installer(install_url, temp_script_path)

            if not download_success:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="下载 Miniconda 安装脚本失败",
                    error="无法从 Anaconda 仓库下载安装脚本"
                )
            
            # 执行安装脚本
            # -b: 批量模式（不需要用户交互）
            # -f: 强制安装（即使目录已存在也不报错）
            install_cmd = f"bash {temp_script_path} -b -f"
            returncode, stdout, stderr = self.run_command(
                install_cmd,
                shell=True,
                timeout=600,  # 安装可能需要较长时间
                max_retries=1
            )
            
            # 清理临时文件
            try:
                Path(temp_script_path).unlink()
            except OSError:
                pass
            
            if returncode != 0:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="执行 Conda 安装脚本失败",
                    error=stderr or "安装脚本返回非零退出码"
                )
            
            # 验证安装
            if not self.verify():
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="Conda 安装验证失败",
                    error="安装脚本执行成功，但无法验证 Conda 是否正确安装"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="conda",
                result=InstallResult.SUCCESS,
                message="Conda 安装成功",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="conda",
                result=InstallResult.FAILED,
                message="Conda 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 Conda
        
        Conda 的升级通过重新下载最新安装包并使用覆盖安装模式实现。
        使用 -b（批量模式）、-u（更新模式）、-p（指定路径）参数。
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="conda",
                result=InstallResult.FAILED,
                message="Conda 未安装，无法升级",
                error="请先安装 Conda"
            )
        
        try:
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 获取安装包 URL
            try:
                install_url = self.get_install_url()
            except ValueError as e:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="不支持的平台",
                    error=str(e)
                )
            
            # 下载安装脚本到临时文件
            with tempfile.NamedTemporaryFile(
                mode='w',
                suffix='.sh',
                delete=False
            ) as temp_file:
                temp_script_path = temp_file.name
            
            # 下载安装脚本（使用 curl 避免 Python SSL 库冲突）
            download_success = self._download_installer(install_url, temp_script_path)

            if not download_success:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="下载 Miniconda 安装脚本失败",
                    error="无法从 Anaconda 仓库下载安装脚本"
                )
            
            # 执行升级（覆盖安装）
            # -b: 批量模式（不需要用户交互）
            # -f: 强制安装（覆盖已有安装）
            upgrade_cmd = f"bash {temp_script_path} -b -f"
            returncode, stdout, stderr = self.run_command(
                upgrade_cmd,
                shell=True,
                timeout=600,
                max_retries=1
            )
            
            # 清理临时文件
            try:
                Path(temp_script_path).unlink()
            except OSError:
                pass
            
            if returncode != 0:
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="执行 Conda 升级脚本失败",
                    error=stderr or "升级脚本返回非零退出码"
                )
            
            # 验证升级
            if not self.verify():
                return InstallReport(
                    tool_name="conda",
                    result=InstallResult.FAILED,
                    message="Conda 升级验证失败",
                    error="升级脚本执行成功，但无法验证 Conda 是否正确安装"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="conda",
                result=InstallResult.SUCCESS,
                message=f"Conda 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="conda",
                result=InstallResult.FAILED,
                message="Conda 升级过程中发生异常",
                error=str(e)
            )



"""镜像源配置模块

该模块负责配置各工具的镜像源，包括 npm、Bun 和 uv。
"""

import subprocess
from pathlib import Path
from typing import Dict, Optional
import tomllib

from .config import RegistryConfig


class MirrorConfigurator:
    """镜像源配置器
    
    负责配置 npm、Bun 和 uv 的镜像源，以加速中国大陆地区的下载速度。
    """
    
    def __init__(self, registry_config: RegistryConfig):
        """初始化镜像源配置器
        
        Args:
            registry_config: 镜像源配置对象
        """
        self.registry_config = registry_config
    
    def configure_npm_mirror(self) -> bool:
        """配置 npm 镜像源
        
        使用 npm config set 命令配置 registry。
        
        Returns:
            配置是否成功
        """
        try:
            # 设置 npm registry
            result = subprocess.run(
                ["npm", "config", "set", "registry", self.registry_config.npm],
                capture_output=True,
                text=True,
                timeout=10
            )
            return result.returncode == 0
        except (subprocess.TimeoutExpired, FileNotFoundError, Exception):
            return False
    
    def configure_bun_mirror(self) -> bool:
        """配置 Bun 镜像源
        
        创建 ~/.bunfig.toml 配置文件并设置 registry。
        
        Returns:
            配置是否成功
        """
        try:
            bunfig_path = Path.home() / ".bunfig.toml"
            
            # 读取现有配置（如果存在）
            existing_config = {}
            if bunfig_path.exists():
                with open(bunfig_path, 'rb') as f:
                    existing_config = tomllib.load(f)
            
            # 构建新配置
            # 保留现有配置，只更新 install.registry
            config_lines = []
            
            # 如果有其他配置，先写入
            for section, values in existing_config.items():
                if section != "install":
                    config_lines.append(f"[{section}]")
                    if isinstance(values, dict):
                        for key, value in values.items():
                            if isinstance(value, str):
                                config_lines.append(f'{key} = "{value}"')
                            elif isinstance(value, bool):
                                # 使用小写的 true/false
                                config_lines.append(f'{key} = {str(value).lower()}')
                            else:
                                config_lines.append(f'{key} = {value}')
                    config_lines.append("")
            
            # 写入 install 配置
            config_lines.append("[install]")
            config_lines.append(f'registry = "{self.registry_config.bun}"')
            
            # 如果 install 中有其他配置，也保留
            if "install" in existing_config:
                for key, value in existing_config["install"].items():
                    if key != "registry":
                        if isinstance(value, str):
                            config_lines.append(f'{key} = "{value}"')
                        elif isinstance(value, bool):
                            # 使用小写的 true/false
                            config_lines.append(f'{key} = {str(value).lower()}')
                        else:
                            config_lines.append(f'{key} = {value}')
            
            config_lines.append("")
            
            # 写入配置文件
            bunfig_path.parent.mkdir(parents=True, exist_ok=True)
            with open(bunfig_path, 'w', encoding='utf-8') as f:
                f.write('\n'.join(config_lines))
            
            return True
        except Exception:
            return False
    
    def configure_uv_mirror(self) -> bool:
        """配置 uv PyPI 镜像源和 CPython 下载代理
        
        创建 ~/.config/uv/uv.toml 配置文件，包含：
        1. python-install-mirror 配置（在 [[index]] 之前）
        2. PyPI 镜像源配置
        
        Returns:
            配置是否成功
        """
        try:
            uv_config_dir = Path.home() / ".config" / "uv"
            uv_config_path = uv_config_dir / "uv.toml"
            
            # 创建配置目录
            uv_config_dir.mkdir(parents=True, exist_ok=True)
            
            # 构建配置内容
            # 注意：python-install-mirror 必须在 [[index]] 之前
            config_lines = [
                "# uv 配置文件",
                "# 由 mono-kickstart 自动生成",
                "",
                "# CPython 下载代理",
                f'python-install-mirror = "{self.registry_config.python_install}"',
                "",
                "# PyPI 镜像源",
                "[[index]]",
                f'url = "{self.registry_config.pypi}"',
                ""
            ]
            
            # 写入配置文件
            with open(uv_config_path, 'w', encoding='utf-8') as f:
                f.write('\n'.join(config_lines))
            
            return True
        except Exception:
            return False
    
    def verify_npm_mirror(self) -> bool:
        """验证 npm 镜像源配置
        
        通过 npm get registry 命令验证配置是否生效。
        
        Returns:
            验证是否成功
        """
        try:
            result = subprocess.run(
                ["npm", "get", "registry"],
                capture_output=True,
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # 检查输出是否包含配置的镜像源
                output = result.stdout.strip()
                # 移除末尾的斜杠进行比较
                configured = self.registry_config.npm.rstrip('/')
                actual = output.rstrip('/')
                return configured == actual
            return False
        except (subprocess.TimeoutExpired, FileNotFoundError, Exception):
            return False
    
    def verify_bun_mirror(self) -> bool:
        """验证 Bun 镜像源配置
        
        检查 ~/.bunfig.toml 文件是否存在且包含正确的配置。
        
        Returns:
            验证是否成功
        """
        try:
            bunfig_path = Path.home() / ".bunfig.toml"
            
            if not bunfig_path.exists():
                return False
            
            # 读取配置文件
            with open(bunfig_path, 'rb') as f:
                config = tomllib.load(f)
            
            # 检查 registry 配置
            registry = config.get("install", {}).get("registry")
            if registry:
                # 移除末尾的斜杠进行比较
                configured = self.registry_config.bun.rstrip('/')
                actual = registry.rstrip('/')
                return configured == actual
            
            return False
        except Exception:
            return False
    
    def verify_uv_mirror(self) -> bool:
        """验证 uv 镜像源配置
        
        检查 ~/.config/uv/uv.toml 文件是否存在且包含正确的配置。
        
        Returns:
            验证是否成功
        """
        try:
            uv_config_path = Path.home() / ".config" / "uv" / "uv.toml"
            
            if not uv_config_path.exists():
                return False
            
            # 读取配置文件内容
            with open(uv_config_path, 'r', encoding='utf-8') as f:
                content = f.read()
            
            # 检查是否包含 python-install-mirror 配置
            if f'python-install-mirror = "{self.registry_config.python_install}"' not in content:
                return False
            
            # 检查是否包含 PyPI 镜像源配置
            if f'url = "{self.registry_config.pypi}"' not in content:
                return False
            
            # 检查 python-install-mirror 是否在 [[index]] 之前
            python_install_pos = content.find('python-install-mirror')
            index_pos = content.find('[[index]]')
            
            if python_install_pos == -1 or index_pos == -1:
                return False
            
            return python_install_pos < index_pos
        except Exception:
            return False
    
    def configure_all(self) -> Dict[str, bool]:
        """配置所有镜像源
        
        依次配置 npm、Bun 和 uv 的镜像源。
        
        Returns:
            字典，键为工具名称，值为配置是否成功
        """
        results = {}
        
        results["npm"] = self.configure_npm_mirror()
        results["bun"] = self.configure_bun_mirror()
        results["uv"] = self.configure_uv_mirror()
        
        return results



"""
CLI 入口模块

定义命令行接口和子命令（使用 argparse 标准库）。
"""

import argparse
import logging
import os
import sys
from pathlib import Path
from typing import Optional

from mono_kickstart import __version__


# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format='%(message)s',
    handlers=[logging.StreamHandler(sys.stdout)]
)
logger = logging.getLogger(__name__)


# 可用工具列表（用于补全）
AVAILABLE_TOOLS = [
    "nvm",
    "node",
    "conda",
    "bun",
    "uv",
    "claude-code",
    "codex",
    "spec-kit",
    "bmad-method",
]


class ChineseHelpFormatter(argparse.RawDescriptionHelpFormatter):
    """中文化的帮助信息格式器"""
    
    def _format_usage(self, usage, actions, groups, prefix):
        if prefix is None:
            prefix = '用法: '
        return super()._format_usage(usage, actions, groups, prefix)


def create_parser() -> argparse.ArgumentParser:
    """创建主解析器和子命令解析器
    
    Returns:
        配置好的 ArgumentParser 对象
    """
    # 主解析器
    parser = argparse.ArgumentParser(
        prog='mk',
        description='Mono-Kickstart - Monorepo 项目模板脚手架 CLI 工具\n\n'
                    '通过一条命令快速初始化标准化的 Monorepo 工程，\n'
                    '自动完成开发环境搭建与工具链安装。',
        formatter_class=ChineseHelpFormatter,
        add_help=True,
    )
    
    parser.add_argument(
        '--version',
        action='version',
        version=f'Mono-Kickstart version {__version__}',
        help='显示版本号'
    )
    
    # 子命令解析器
    subparsers = parser.add_subparsers(
        title='可用命令',
        dest='command',
        help='子命令帮助信息'
    )
    
    # init 子命令
    init_parser = subparsers.add_parser(
        'init',
        help='初始化 Monorepo 项目和开发环境',
        description='初始化 Monorepo 项目和开发环境',
        formatter_class=ChineseHelpFormatter,
    )
    init_parser.add_argument(
        '--config',
        type=str,
        metavar='PATH',
        help='配置文件路径'
    )
    init_parser.add_argument(
        '--save-config',
        action='store_true',
        help='保存配置到 .kickstartrc'
    )
    init_parser.add_argument(
        '--interactive',
        action='store_true',
        help='交互式配置'
    )
    init_parser.add_argument(
        '--force',
        action='store_true',
        help='强制覆盖已有配置'
    )
    init_parser.add_argument(
        '--dry-run',
        action='store_true',
        help='模拟运行，不实际安装'
    )
    
    # upgrade 子命令
    upgrade_parser = subparsers.add_parser(
        'upgrade',
        help='升级已安装的开发工具',
        description='升级已安装的开发工具',
        formatter_class=ChineseHelpFormatter,
    )
    upgrade_parser.add_argument(
        'tool',
        nargs='?',
        choices=AVAILABLE_TOOLS,
        metavar='TOOL',
        help=f'要升级的工具名称 (可选值: {", ".join(AVAILABLE_TOOLS)})'
    )
    upgrade_parser.add_argument(
        '--all',
        action='store_true',
        help='升级所有工具'
    )
    upgrade_parser.add_argument(
        '--dry-run',
        action='store_true',
        help='模拟运行，不实际升级'
    )
    
    # install 子命令
    install_parser = subparsers.add_parser(
        'install',
        help='安装开发工具',
        description='安装开发工具',
        formatter_class=ChineseHelpFormatter,
    )
    install_parser.add_argument(
        'tool',
        nargs='?',
        choices=AVAILABLE_TOOLS,
        metavar='TOOL',
        help=f'要安装的工具名称 (可选值: {", ".join(AVAILABLE_TOOLS)})'
    )
    install_parser.add_argument(
        '--all',
        action='store_true',
        help='安装所有工具'
    )
    install_parser.add_argument(
        '--dry-run',
        action='store_true',
        help='模拟运行，不实际安装'
    )
    
    # setup-shell 子命令
    setup_shell_parser = subparsers.add_parser(
        'setup-shell',
        help='配置 shell（PATH 和 Tab 补全）',
        description='配置 shell（PATH 和 Tab 补全）',
        formatter_class=ChineseHelpFormatter,
    )
    
    return parser


def cmd_init(args: argparse.Namespace) -> int:
    """执行 init 命令
    
    Args:
        args: 解析后的命令行参数
        
    Returns:
        退出码（0 表示成功）
    """
    from mono_kickstart.platform_detector import PlatformDetector
    from mono_kickstart.config import ConfigManager
    from mono_kickstart.orchestrator import InstallOrchestrator
    
    logger.info("🚀 Mono-Kickstart - 初始化 Monorepo 项目")
    logger.info("")
    
    try:
        # 1. 检测平台
        logger.info("📋 检测平台信息...")
        detector = PlatformDetector()
        
        if not detector.is_supported():
            platform_info = detector.detect_all()
            logger.error(f"❌ 错误: 不支持的平台 ({platform_info.os.value}/{platform_info.arch.value})")
            logger.error("支持的平台:")
            logger.error("  - macOS ARM64")
            logger.error("  - macOS x86_64")
            logger.error("  - Linux x86_64")
            return 1
        
        platform_info = detector.detect_all()
        logger.info(f"✓ 平台: {platform_info.os.value}/{platform_info.arch.value}")
        logger.info(f"✓ Shell: {platform_info.shell.value}")
        logger.info("")
        
        # 2. 加载配置
        config_manager = ConfigManager()
        
        # 如果使用交互式模式
        if args.interactive:
            from mono_kickstart.interactive import InteractiveConfigurator
            
            # 加载默认配置作为交互式配置的基础
            try:
                default_config = config_manager.load_with_priority(
                    cli_config=Path(args.config) if args.config else None,
                    project_config=Path(".kickstartrc"),
                    user_config=Path.home() / ".kickstartrc"
                )
            except Exception:
                # 如果加载失败，使用空配置
                default_config = config_manager.load_from_defaults()
            
            # 运行交互式配置向导
            configurator = InteractiveConfigurator(default_config)
            config = configurator.run_wizard()
            
            # 显示配置摘要并确认
            if not configurator.confirm_config(config):
                logger.info("❌ 用户取消操作")
                return 0
            
            logger.info("")
        else:
            # 非交互式模式：按优先级加载配置
            logger.info("📋 加载配置...")
            
            try:
                cli_config_path = Path(args.config) if args.config else None
                config = config_manager.load_with_priority(
                    cli_config=cli_config_path,
                    project_config=Path(".kickstartrc"),
                    user_config=Path.home() / ".kickstartrc"
                )
                
                # 验证配置
                errors = config_manager.validate(config)
                if errors:
                    logger.error("❌ 配置验证失败:")
                    for error in errors:
                        logger.error(f"  - {error}")
                    return 2
                
                logger.info("✓ 配置加载成功")
                logger.info("")
                
            except FileNotFoundError as e:
                logger.error(f"❌ 配置文件不存在: {e}")
                return 2
            except Exception as e:
                logger.error(f"❌ 配置加载失败: {e}")
                logger.debug("详细错误信息:", exc_info=True)
                return 2
        
        # 3. 保存配置（如果需要）
        if args.save_config:
            try:
                config_path = Path(".kickstartrc")
                config_manager.save_to_file(config, config_path)
                logger.info(f"✓ 配置已保存到 {config_path}")
                logger.info("")
            except Exception as e:
                logger.warning(f"⚠️  警告: 配置保存失败: {e}")
                logger.info("")
        
        # 4. 创建安装编排器
        orchestrator = InstallOrchestrator(
            config=config,
            platform_info=platform_info,
            dry_run=args.dry_run
        )
        
        # 5. 执行初始化流程
        if args.dry_run:
            logger.info("🔍 [模拟运行模式]")
            logger.info("")
        
        logger.info("🚀 开始初始化...")
        logger.info("")
        
        # 执行完整初始化流程
        reports = orchestrator.run_init(
            project_name=config.project.name,
            force=args.force
        )
        
        # 打印摘要
        orchestrator.print_summary(reports)
        
        # 检查是否有失败的任务
        from mono_kickstart.installer_base import InstallResult
        failed_count = sum(1 for r in reports.values() if r.result == InstallResult.FAILED)
        
        if failed_count == len(reports):
            # 所有任务都失败
            logger.error("❌ 所有任务都失败了")
            return 3
        elif failed_count > 0:
            # 部分任务失败
            logger.warning(f"⚠️  {failed_count} 个任务失败，但其他任务已成功完成")
            return 0
        else:
            # 全部成功
            logger.info("✨ 初始化完成！")
            return 0
            
    except KeyboardInterrupt:
        logger.error("\n❌ 用户中断操作")
        return 130
    except Exception as e:
        logger.error(f"❌ 初始化过程中发生错误: {e}")
        logger.debug("详细错误信息:", exc_info=True)
        return 1


def cmd_upgrade(args: argparse.Namespace) -> int:
    """执行 upgrade 命令
    
    Args:
        args: 解析后的命令行参数
        
    Returns:
        退出码（0 表示成功）
    """
    from mono_kickstart.platform_detector import PlatformDetector
    from mono_kickstart.config import ConfigManager
    from mono_kickstart.orchestrator import InstallOrchestrator
    from mono_kickstart.tool_detector import ToolDetector
    
    logger.info("🔄 Mono-Kickstart - 升级开发工具")
    logger.info("")
    
    try:
        # 1. 检测平台
        detector = PlatformDetector()
        if not detector.is_supported():
            platform_info = detector.detect_all()
            logger.error(f"❌ 错误: 不支持的平台 ({platform_info.os.value}/{platform_info.arch.value})")
            return 1
        
        platform_info = detector.detect_all()
        
        # 2. 加载配置
        config_manager = ConfigManager()
        try:
            config = config_manager.load_with_priority(
                cli_config=None,
                project_config=Path(".kickstartrc"),
                user_config=Path.home() / ".kickstartrc"
            )
        except Exception as e:
            logger.warning(f"⚠️  警告: 配置加载失败，使用默认配置: {e}")
            config = config_manager.load_from_defaults()
        
        # 3. 创建安装编排器
        orchestrator = InstallOrchestrator(
            config=config,
            platform_info=platform_info,
            dry_run=args.dry_run
        )
        
        # 4. 确定要升级的工具
        if args.dry_run:
            logger.info("🔍 [模拟运行模式]")
            logger.info("")
        
        # 如果指定了 --all 或没有指定工具名称，升级所有已安装的工具
        tool_name = None
        if not args.all and args.tool:
            tool_name = args.tool
            logger.info(f"🔄 升级工具: {tool_name}")
        else:
            logger.info("🔄 升级所有已安装的工具")
            # 检测已安装的工具
            tool_detector = ToolDetector()
            all_tools = tool_detector.detect_all_tools()
            installed_tools = [name for name, status in all_tools.items() if status.installed]
            
            if not installed_tools:
                logger.warning("⚠️  没有检测到已安装的工具")
                return 0
            
            logger.info(f"检测到 {len(installed_tools)} 个已安装的工具:")
            for tool in installed_tools:
                logger.info(f"  - {tool}")
        
        logger.info("")
        
        # 5. 执行升级流程
        reports = orchestrator.run_upgrade(tool_name=tool_name)
        
        # 打印摘要
        orchestrator.print_summary(reports)
        
        # 检查是否有失败的任务
        from mono_kickstart.installer_base import InstallResult
        failed_count = sum(1 for r in reports.values() if r.result == InstallResult.FAILED)
        
        if failed_count == len(reports) and len(reports) > 0:
            # 所有任务都失败
            logger.error("❌ 所有任务都失败了")
            return 3
        elif failed_count > 0:
            # 部分任务失败
            logger.warning(f"⚠️  {failed_count} 个任务失败，但其他任务已成功完成")
            return 0
        else:
            # 全部成功
            logger.info("✨ 升级完成！")
            return 0
            
    except KeyboardInterrupt:
        logger.error("\n❌ 用户中断操作")
        return 130
    except Exception as e:
        logger.error(f"❌ 升级过程中发生错误: {e}")
        logger.debug("详细错误信息:", exc_info=True)
        return 1


def cmd_install(args: argparse.Namespace) -> int:
    """执行 install 命令
    
    Args:
        args: 解析后的命令行参数
        
    Returns:
        退出码（0 表示成功）
    """
    from mono_kickstart.platform_detector import PlatformDetector
    from mono_kickstart.config import ConfigManager
    from mono_kickstart.orchestrator import InstallOrchestrator
    
    logger.info("📦 Mono-Kickstart - 安装开发工具")
    logger.info("")
    
    try:
        # 1. 检测平台
        detector = PlatformDetector()
        if not detector.is_supported():
            platform_info = detector.detect_all()
            logger.error(f"❌ 错误: 不支持的平台 ({platform_info.os.value}/{platform_info.arch.value})")
            return 1
        
        platform_info = detector.detect_all()
        
        # 2. 加载配置
        config_manager = ConfigManager()
        try:
            config = config_manager.load_with_priority(
                cli_config=None,
                project_config=Path(".kickstartrc"),
                user_config=Path.home() / ".kickstartrc"
            )
        except Exception as e:
            logger.warning(f"⚠️  警告: 配置加载失败，使用默认配置: {e}")
            config = config_manager.load_from_defaults()
        
        # 3. 创建安装编排器
        orchestrator = InstallOrchestrator(
            config=config,
            platform_info=platform_info,
            dry_run=args.dry_run
        )
        
        # 4. 确定要安装的工具
        if args.dry_run:
            logger.info("🔍 [模拟运行模式]")
            logger.info("")
        
        if not args.all and not args.tool:
            logger.error("❌ 错误: 请指定要安装的工具名称或使用 --all 安装所有工具")
            return 1
        
        # 5. 执行安装流程
        if args.all:
            # 安装所有工具
            logger.info("📦 安装所有工具")
            logger.info("")
            reports = orchestrator.install_all_tools()
        else:
            # 安装单个工具
            tool_name = args.tool
            logger.info(f"📦 安装工具: {tool_name}")
            logger.info("")
            report = orchestrator.install_tool(tool_name)
            reports = {tool_name: report}
        
        # 打印摘要
        orchestrator.print_summary(reports)
        
        # 检查是否有失败的任务
        from mono_kickstart.installer_base import InstallResult
        failed_count = sum(1 for r in reports.values() if r.result == InstallResult.FAILED)
        
        if failed_count == len(reports) and len(reports) > 0:
            # 所有任务都失败
            logger.error("❌ 所有任务都失败了")
            return 3
        elif failed_count > 0:
            # 部分任务失败
            logger.warning(f"⚠️  {failed_count} 个任务失败，但其他任务已成功完成")
            return 0
        else:
            # 全部成功
            logger.info("✨ 安装完成！")
            return 0
            
    except KeyboardInterrupt:
        logger.error("\n❌ 用户中断操作")
        return 130
    except Exception as e:
        logger.error(f"❌ 安装过程中发生错误: {e}")
        logger.debug("详细错误信息:", exc_info=True)
        return 1


def cmd_setup_shell(args: argparse.Namespace) -> int:
    """执行 setup-shell 命令
    
    配置 shell 环境（PATH 和 Tab 补全）
    
    Args:
        args: 解析后的命令行参数
        
    Returns:
        退出码（0 表示成功）
    """
    from mono_kickstart.shell_completion import setup_shell_completion
    
    try:
        setup_shell_completion()
        return 0
    except Exception as e:
        print(f"错误: {e}", file=sys.stderr)
        return 1


def main() -> int:
    """主入口函数
    
    Returns:
        退出码（0 表示成功，非 0 表示失败）
    """
    parser = create_parser()
    args = parser.parse_args()
    
    # 如果没有指定子命令，显示帮助信息
    if not args.command:
        parser.print_help()
        return 0
    
    # 根据子命令调用相应的处理函数
    if args.command == 'init':
        return cmd_init(args)
    elif args.command == 'upgrade':
        return cmd_upgrade(args)
    elif args.command == 'install':
        return cmd_install(args)
    elif args.command == 'setup-shell':
        return cmd_setup_shell(args)
    else:
        parser.print_help()
        return 1


if __name__ == "__main__":
    sys.exit(main())



"""项目创建器的基于属性的测试

本模块使用 Hypothesis 框架测试项目创建器的通用属性。
"""

import json
import tempfile
from pathlib import Path

from hypothesis import given, settings, strategies as st, HealthCheck

from mono_kickstart.project_creator import ProjectCreator


# 定义测试数据生成策略

@st.composite
def project_name_strategy(draw):
    """生成有效的项目名称策略
    
    项目名称应该是有效的目录名，不包含特殊字符
    """
    # 使用字母、数字、连字符和下划线
    return draw(st.text(
        alphabet=st.characters(
            whitelist_categories=('Lu', 'Ll', 'Nd'),
            whitelist_characters='-_'
        ),
        min_size=1,
        max_size=50
    ).filter(lambda s: s and not s.startswith('-') and not s.startswith('_')))


@st.composite
def project_path_strategy(draw, base_path):
    """生成项目路径策略
    
    Args:
        base_path: 基础临时目录路径
    """
    # 生成相对路径（可能包含子目录）
    depth = draw(st.integers(min_value=0, max_value=2))
    
    if depth == 0:
        return base_path
    
    path_parts = [draw(st.text(
        alphabet=st.characters(
            whitelist_categories=('Lu', 'Ll', 'Nd'),
            whitelist_characters='-_'
        ),
        min_size=1,
        max_size=20
    ).filter(lambda s: s and not s.startswith('-'))) for _ in range(depth)]
    
    result_path = base_path
    for part in path_parts:
        result_path = result_path / part
    
    return result_path


class TestProjectDirectoryStructureConsistency:
    """测试属性 14: 项目目录结构一致性
    
    **Validates: Requirements 3.1**
    """
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_standard_directories_always_created(self, project_name):
        """验证标准目录总是被创建
        
        对于任何项目名称，项目创建器应该生成包含标准目录
        (apps/, packages/, shared/) 的项目结构。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            assert error is None
            
            # 验证标准目录存在
            assert (project_path / "apps").exists(), "apps/ directory not created"
            assert (project_path / "apps").is_dir(), "apps/ is not a directory"
            
            assert (project_path / "packages").exists(), "packages/ directory not created"
            assert (project_path / "packages").is_dir(), "packages/ is not a directory"
            
            assert (project_path / "shared").exists(), "shared/ directory not created"
            assert (project_path / "shared").is_dir(), "shared/ is not a directory"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_standard_files_always_created(self, project_name):
        """验证标准文件总是被创建
        
        对于任何项目名称，项目创建器应该生成标准文件
        (.gitignore, README.md, package.json)。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 验证标准文件存在
            assert (project_path / ".gitignore").exists(), ".gitignore not created"
            assert (project_path / ".gitignore").is_file(), ".gitignore is not a file"
            
            assert (project_path / "README.md").exists(), "README.md not created"
            assert (project_path / "README.md").is_file(), "README.md is not a file"
            
            assert (project_path / "package.json").exists(), "package.json not created"
            assert (project_path / "package.json").is_file(), "package.json is not a file"
            
            assert (project_path / "pnpm-workspace.yaml").exists(), "pnpm-workspace.yaml not created"
            assert (project_path / "pnpm-workspace.yaml").is_file(), "pnpm-workspace.yaml is not a file"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_gitkeep_files_in_empty_directories(self, project_name):
        """验证空目录包含 .gitkeep 文件
        
        对于任何项目名称，标准目录应该包含 .gitkeep 文件
        以确保空目录被 Git 跟踪。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 验证 .gitkeep 文件存在
            assert (project_path / "apps" / ".gitkeep").exists(), "apps/.gitkeep not created"
            assert (project_path / "packages" / ".gitkeep").exists(), "packages/.gitkeep not created"
            assert (project_path / "shared" / ".gitkeep").exists(), "shared/.gitkeep not created"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_package_json_structure_consistency(self, project_name):
        """验证 package.json 结构一致性
        
        对于任何项目名称，package.json 应该包含标准字段
        (name, version, private, workspaces, scripts)。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 读取 package.json
            package_json_path = project_path / "package.json"
            with open(package_json_path, "r", encoding="utf-8") as f:
                package_json = json.load(f)
            
            # 验证标准字段存在
            assert "name" in package_json, "package.json missing 'name' field"
            assert package_json["name"] == project_name, "package.json name doesn't match project name"
            
            assert "version" in package_json, "package.json missing 'version' field"
            assert isinstance(package_json["version"], str), "version should be a string"
            
            assert "private" in package_json, "package.json missing 'private' field"
            assert package_json["private"] is True, "package.json should be private"
            
            assert "workspaces" in package_json, "package.json missing 'workspaces' field"
            assert isinstance(package_json["workspaces"], list), "workspaces should be a list"
            assert "apps/*" in package_json["workspaces"], "workspaces missing 'apps/*'"
            assert "packages/*" in package_json["workspaces"], "workspaces missing 'packages/*'"
            
            assert "scripts" in package_json, "package.json missing 'scripts' field"
            assert isinstance(package_json["scripts"], dict), "scripts should be a dict"
            assert "dev" in package_json["scripts"], "scripts missing 'dev'"
            assert "build" in package_json["scripts"], "scripts missing 'build'"
            assert "test" in package_json["scripts"], "scripts missing 'test'"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_pnpm_workspace_structure_consistency(self, project_name):
        """验证 pnpm-workspace.yaml 结构一致性
        
        对于任何项目名称，pnpm-workspace.yaml 应该包含
        标准的 workspace 配置。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 读取 pnpm-workspace.yaml
            pnpm_workspace_path = project_path / "pnpm-workspace.yaml"
            with open(pnpm_workspace_path, "r", encoding="utf-8") as f:
                content = f.read()
            
            # 验证包含标准配置
            assert "packages:" in content, "pnpm-workspace.yaml missing 'packages:'"
            assert "'apps/*'" in content, "pnpm-workspace.yaml missing 'apps/*'"
            assert "'packages/*'" in content, "pnpm-workspace.yaml missing 'packages/*'"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_gitignore_contains_standard_patterns(self, project_name):
        """验证 .gitignore 包含标准忽略模式
        
        对于任何项目名称，.gitignore 应该包含常见的
        忽略模式（node_modules, dist, .env 等）。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 读取 .gitignore
            gitignore_path = project_path / ".gitignore"
            with open(gitignore_path, "r", encoding="utf-8") as f:
                content = f.read()
            
            # 验证包含标准忽略模式
            standard_patterns = [
                "node_modules/",
                "dist/",
                "build/",
                ".env",
                ".DS_Store",
                "__pycache__/",
                "*.log"
            ]
            
            for pattern in standard_patterns:
                assert pattern in content, f".gitignore missing pattern: {pattern}"
    
    @given(project_name=project_name_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_readme_contains_project_name(self, project_name):
        """验证 README.md 包含项目名称
        
        对于任何项目名称，README.md 应该包含该项目名称。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            success, error = creator.create_project()
            
            # 验证创建成功
            assert success is True, f"Project creation failed: {error}"
            
            # 读取 README.md
            readme_path = project_path / "README.md"
            with open(readme_path, "r", encoding="utf-8") as f:
                content = f.read()
            
            # 验证包含项目名称
            assert project_name in content, f"README.md doesn't contain project name: {project_name}"
    
    @given(
        project_name=project_name_strategy(),
        iterations=st.integers(min_value=2, max_value=3)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_idempotent_creation(self, project_name, iterations):
        """验证项目创建的幂等性
        
        对于任何项目名称，多次创建项目（使用 force=True）
        应该产生相同的目录结构。
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            project_path = tmp_path / project_name
            
            creator = ProjectCreator(project_name, project_path)
            
            # 多次创建项目
            for i in range(iterations):
                success, error = creator.create_project(force=True)
                assert success is True, f"Project creation failed on iteration {i}: {error}"
            
            # 验证最终结构仍然正确
            assert (project_path / "apps").exists()
            assert (project_path / "packages").exists()
            assert (project_path / "shared").exists()
            assert (project_path / "package.json").exists()
            assert (project_path / "README.md").exists()
            
            # 验证 package.json 仍然有效
            with open(project_path / "package.json", "r", encoding="utf-8") as f:
                package_json = json.load(f)
            assert package_json["name"] == project_name
    
    @given(
        project_name1=project_name_strategy(),
        project_name2=project_name_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_different_names_same_structure(self, project_name1, project_name2):
        """验证不同项目名称产生相同的目录结构
        
        对于任何两个不同的项目名称，应该产生相同的目录结构
        （除了项目名称本身）。
        """
        # 跳过相同名称的情况
        if project_name1 == project_name2:
            return
        
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            
            # 创建第一个项目
            project_path1 = tmp_path / project_name1
            creator1 = ProjectCreator(project_name1, project_path1)
            success1, error1 = creator1.create_project()
            assert success1 is True, f"Project 1 creation failed: {error1}"
            
            # 创建第二个项目
            project_path2 = tmp_path / project_name2
            creator2 = ProjectCreator(project_name2, project_path2)
            success2, error2 = creator2.create_project()
            assert success2 is True, f"Project 2 creation failed: {error2}"
            
            # 验证两个项目有相同的目录结构
            dirs_to_check = ["apps", "packages", "shared"]
            for dir_name in dirs_to_check:
                assert (project_path1 / dir_name).exists(), f"Project 1 missing {dir_name}"
                assert (project_path2 / dir_name).exists(), f"Project 2 missing {dir_name}"
            
            # 验证两个项目有相同的文件
            files_to_check = [".gitignore", "README.md", "package.json", "pnpm-workspace.yaml"]
            for file_name in files_to_check:
                assert (project_path1 / file_name).exists(), f"Project 1 missing {file_name}"
                assert (project_path2 / file_name).exists(), f"Project 2 missing {file_name}"
            
            # 验证 package.json 结构相同（除了 name 字段）
            with open(project_path1 / "package.json", "r", encoding="utf-8") as f:
                package1 = json.load(f)
            with open(project_path2 / "package.json", "r", encoding="utf-8") as f:
                package2 = json.load(f)
            
            # 比较除 name 外的所有字段
            assert package1["version"] == package2["version"]
            assert package1["private"] == package2["private"]
            assert package1["workspaces"] == package2["workspaces"]
            assert package1["scripts"] == package2["scripts"]



"""Claude Code CLI 安装器模块

该模块实现 Claude Code CLI 的安装、升级和验证逻辑。
"""

import shutil
from pathlib import Path
from typing import Optional

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo


class ClaudeCodeInstaller(ToolInstaller):
    """Claude Code CLI 安装器
    
    负责安装、升级和验证 Claude Code CLI。
    Claude Code CLI 通过原生二进制安装方式（stable 渠道）进行安装。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        install_script_url: 安装脚本 URL
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 Claude Code CLI 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # Claude Code CLI 官方安装脚本 URL
        self.install_script_url = "https://claude.ai/install.sh"
    
    def verify(self) -> bool:
        """验证 Claude Code CLI 是否已正确安装

        检查 claude 命令是否可用，并尝试执行 claude --version 命令。

        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        # 检查 claude 命令是否在 PATH 中
        if not shutil.which("claude"):
            return False

        # 尝试执行 claude --version 命令进行验证
        returncode, stdout, stderr = self.run_command(
            "claude --version",
            shell=True,
            timeout=30,
            max_retries=1
        )

        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 Claude Code CLI 版本
        
        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        if not shutil.which("claude"):
            return None
        
        returncode, stdout, stderr = self.run_command(
            "claude --version",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        if returncode == 0:
            return stdout.strip()
        
        return None
    
    def install(self) -> InstallReport:
        """安装 Claude Code CLI
        
        执行以下步骤：
        1. 检查 Claude Code CLI 是否已安装
        2. 执行官方安装脚本（stable 渠道）
        3. 使用 claude --version 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.SKIPPED,
                message="Claude Code CLI 已安装，跳过安装",
                version=version
            )
        
        try:
            # 执行官方安装脚本
            # curl -fsSL https://claude.ai/download/cli/install.sh | sh
            install_cmd = f"curl -fsSL {self.install_script_url} | bash"
            returncode, stdout, stderr = self.run_command(
                install_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )

            if returncode != 0:
                return InstallReport(
                    tool_name="claude-code",
                    result=InstallResult.FAILED,
                    message="执行 Claude Code CLI 安装脚本失败",
                    error=stderr or "安装脚本返回非零退出码"
                )

            # 验证安装（使用 claude --version）
            if not self.verify():
                return InstallReport(
                    tool_name="claude-code",
                    result=InstallResult.FAILED,
                    message="Claude Code CLI 安装验证失败",
                    error="安装脚本执行成功，但 claude --version 验证失败"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.SUCCESS,
                message="Claude Code CLI 安装成功",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.FAILED,
                message="Claude Code CLI 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 Claude Code CLI
        
        Claude Code CLI 的升级通过重新执行官方安装脚本实现。
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.FAILED,
                message="Claude Code CLI 未安装，无法升级",
                error="请先安装 Claude Code CLI"
            )
        
        try:
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 执行官方安装脚本（会自动升级）
            install_cmd = f"curl -fsSL {self.install_script_url} | bash"
            returncode, stdout, stderr = self.run_command(
                install_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )
            
            if returncode != 0:
                return InstallReport(
                    tool_name="claude-code",
                    result=InstallResult.FAILED,
                    message="执行 Claude Code CLI 升级脚本失败",
                    error=stderr or "升级脚本返回非零退出码"
                )
            
            # 验证升级（使用 claude --version）
            if not self.verify():
                return InstallReport(
                    tool_name="claude-code",
                    result=InstallResult.FAILED,
                    message="Claude Code CLI 升级验证失败",
                    error="升级脚本执行成功，但 claude --version 验证失败"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.SUCCESS,
                message=f"Claude Code CLI 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="claude-code",
                result=InstallResult.FAILED,
                message="Claude Code CLI 升级过程中发生异常",
                error=str(e)
            )



"""错误处理模块

定义自定义异常类和错误恢复建议。
"""

from typing import Optional, List
from enum import Enum


class ExitCode(Enum):
    """退出码枚举"""
    SUCCESS = 0
    GENERAL_ERROR = 1
    CONFIG_ERROR = 2
    ALL_TASKS_FAILED = 3
    PERMISSION_ERROR = 4
    DEPENDENCY_ERROR = 5
    USER_INTERRUPT = 130


class MonoKickstartError(Exception):
    """Mono-Kickstart 基础异常类"""
    
    def __init__(
        self,
        message: str,
        exit_code: ExitCode = ExitCode.GENERAL_ERROR,
        recovery_suggestions: Optional[List[str]] = None
    ):
        """初始化异常
        
        Args:
            message: 错误消息
            exit_code: 退出码
            recovery_suggestions: 恢复建议列表
        """
        super().__init__(message)
        self.message = message
        self.exit_code = exit_code
        self.recovery_suggestions = recovery_suggestions or []


class PlatformNotSupportedError(MonoKickstartError):
    """平台不支持错误"""
    
    def __init__(self, os: str, arch: str):
        """初始化平台不支持错误
        
        Args:
            os: 操作系统
            arch: 架构
        """
        message = f"不支持的平台: {os}/{arch}"
        recovery_suggestions = [
            "支持的平台:",
            "  - macOS ARM64",
            "  - macOS x86_64",
            "  - Linux x86_64",
            "请在支持的平台上运行此工具。"
        ]
        super().__init__(
            message=message,
            exit_code=ExitCode.GENERAL_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.os = os
        self.arch = arch


class PythonVersionError(MonoKickstartError):
    """Python 版本不满足错误"""
    
    def __init__(self, current_version: str, required_version: str = "3.11"):
        """初始化 Python 版本错误
        
        Args:
            current_version: 当前 Python 版本
            required_version: 要求的 Python 版本
        """
        message = f"Python 版本不满足要求: 当前 {current_version}, 需要 {required_version}+"
        recovery_suggestions = [
            f"请升级 Python 到 {required_version} 或更高版本。",
            "推荐使用以下方式安装 Python:",
            "  - 使用 pyenv: pyenv install 3.11",
            "  - 使用 Conda: conda install python=3.11",
            "  - 从官网下载: https://www.python.org/downloads/"
        ]
        super().__init__(
            message=message,
            exit_code=ExitCode.GENERAL_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.current_version = current_version
        self.required_version = required_version


class ConfigError(MonoKickstartError):
    """配置错误"""
    
    def __init__(self, message: str, config_file: Optional[str] = None):
        """初始化配置错误
        
        Args:
            message: 错误消息
            config_file: 配置文件路径
        """
        full_message = f"配置错误: {message}"
        if config_file:
            full_message += f" (文件: {config_file})"
        
        recovery_suggestions = [
            "请检查配置文件格式是否正确。",
            "配置文件应为有效的 YAML 格式。",
            "参考文档: https://github.com/mono-kickstart/mono-kickstart#configuration"
        ]
        
        super().__init__(
            message=full_message,
            exit_code=ExitCode.CONFIG_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.config_file = config_file


class ToolInstallError(MonoKickstartError):
    """工具安装错误"""
    
    def __init__(
        self,
        tool_name: str,
        reason: str,
        manual_install_guide: Optional[str] = None
    ):
        """初始化工具安装错误
        
        Args:
            tool_name: 工具名称
            reason: 失败原因
            manual_install_guide: 手动安装指引
        """
        message = f"工具 '{tool_name}' 安装失败: {reason}"
        
        recovery_suggestions = []
        if manual_install_guide:
            recovery_suggestions.append(f"手动安装指引: {manual_install_guide}")
        
        recovery_suggestions.extend([
            "您可以尝试:",
            "  1. 检查网络连接",
            "  2. 重新运行安装命令",
            "  3. 手动安装该工具",
            "  4. 跳过该工具继续安装其他工具"
        ])
        
        super().__init__(
            message=message,
            exit_code=ExitCode.GENERAL_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.tool_name = tool_name
        self.reason = reason


class NetworkError(MonoKickstartError):
    """网络错误"""
    
    def __init__(self, url: str, reason: str):
        """初始化网络错误
        
        Args:
            url: 请求的 URL
            reason: 失败原因
        """
        message = f"网络请求失败: {url} - {reason}"
        recovery_suggestions = [
            "请检查网络连接。",
            "如果在中国大陆，建议使用 --interactive 模式配置镜像源。",
            "您也可以稍后重试。"
        ]
        
        super().__init__(
            message=message,
            exit_code=ExitCode.GENERAL_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.url = url
        self.reason = reason


class PermissionError(MonoKickstartError):
    """权限错误"""
    
    def __init__(self, path: str, operation: str):
        """初始化权限错误
        
        Args:
            path: 文件或目录路径
            operation: 操作类型（读、写、执行等）
        """
        message = f"权限不足: 无法{operation} {path}"
        recovery_suggestions = [
            "请检查文件或目录的权限。",
            "您可能需要使用 sudo 运行命令（不推荐）。",
            "或者更改文件/目录的所有者或权限。"
        ]
        
        super().__init__(
            message=message,
            exit_code=ExitCode.PERMISSION_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.path = path
        self.operation = operation


class DependencyError(MonoKickstartError):
    """依赖缺失错误"""
    
    def __init__(self, dependency: str, required_by: str, install_command: Optional[str] = None):
        """初始化依赖错误
        
        Args:
            dependency: 缺失的依赖
            required_by: 需要该依赖的工具
            install_command: 安装命令
        """
        message = f"缺少依赖: {dependency} (被 {required_by} 需要)"
        
        recovery_suggestions = []
        if install_command:
            recovery_suggestions.append(f"安装命令: {install_command}")
        
        recovery_suggestions.extend([
            f"请先安装 {dependency}，然后重新运行。",
            "或者跳过 {required_by} 的安装。"
        ])
        
        super().__init__(
            message=message,
            exit_code=ExitCode.DEPENDENCY_ERROR,
            recovery_suggestions=recovery_suggestions
        )
        self.dependency = dependency
        self.required_by = required_by


def format_error_message(error: MonoKickstartError) -> str:
    """格式化错误消息
    
    Args:
        error: 错误对象
        
    Returns:
        格式化后的错误消息
    """
    lines = [
        f"❌ 错误: {error.message}",
        ""
    ]
    
    if error.recovery_suggestions:
        lines.append("💡 恢复建议:")
        for suggestion in error.recovery_suggestions:
            lines.append(f"   {suggestion}")
        lines.append("")
    
    return "\n".join(lines)


def handle_error(error: Exception, logger) -> int:
    """统一错误处理
    
    Args:
        error: 异常对象
        logger: 日志记录器
        
    Returns:
        退出码
    """
    if isinstance(error, MonoKickstartError):
        # 自定义错误
        logger.error(format_error_message(error))
        return error.exit_code.value
    elif isinstance(error, KeyboardInterrupt):
        # 用户中断
        logger.error("\n❌ 用户中断操作")
        return ExitCode.USER_INTERRUPT.value
    else:
        # 未知错误
        logger.error(f"❌ 发生未知错误: {error}")
        logger.debug("详细错误信息:", exc_info=True)
        return ExitCode.GENERAL_ERROR.value



"""Codex CLI 安装器模块

该模块实现 Codex CLI 的安装、升级和验证逻辑。
"""

import shutil
from pathlib import Path
from typing import Optional, Literal

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo


class CodexInstaller(ToolInstaller):
    """Codex CLI 安装器
    
    负责安装、升级和验证 Codex CLI。
    Codex CLI 优先使用 Bun 安装（如果 Bun 已安装），否则使用 npm 安装。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        install_method: 安装方式（'bun' 或 'npm'）
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 Codex CLI 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # 确定安装方式
        self.install_method = self._determine_install_method()
    
    def _determine_install_method(self) -> Literal['bun', 'npm']:
        """确定安装方式
        
        如果配置中指定了 install_via，则使用指定的方式。
        否则，如果 Bun 已安装则使用 Bun，否则使用 npm。
        
        Returns:
            Literal['bun', 'npm']: 安装方式
        """
        # 如果配置中指定了安装方式，则使用指定的方式
        if self.config.install_via:
            return self.config.install_via.lower()
        
        # 检查 Bun 是否已安装
        if shutil.which("bun"):
            return 'bun'
        
        # 默认使用 npm
        return 'npm'
    
    def verify(self) -> bool:
        """验证 Codex CLI 是否已正确安装
        
        检查 codex 命令是否可用，并尝试执行 codex --version 命令。
        
        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        # 检查 codex 命令是否在 PATH 中
        if not shutil.which("codex"):
            return False
        
        # 尝试执行 codex --version 命令
        returncode, stdout, stderr = self.run_command(
            "codex --version",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 Codex CLI 版本
        
        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        if not shutil.which("codex"):
            return None
        
        returncode, stdout, stderr = self.run_command(
            "codex --version",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        if returncode == 0:
            return stdout.strip()
        
        return None
    
    def install(self) -> InstallReport:
        """安装 Codex CLI
        
        执行以下步骤：
        1. 检查 Codex CLI 是否已安装
        2. 确定安装方式（Bun 优先，否则 npm）
        3. 执行相应的安装命令
        4. 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="codex",
                result=InstallResult.SKIPPED,
                message="Codex CLI 已安装，跳过安装",
                version=version
            )
        
        try:
            # 根据安装方式执行相应的安装命令
            if self.install_method == 'bun':
                # 检查 Bun 是否可用
                if not shutil.which("bun"):
                    return InstallReport(
                        tool_name="codex",
                        result=InstallResult.FAILED,
                        message="Bun 未安装，无法使用 Bun 安装 Codex CLI",
                        error="请先安装 Bun 或使用 npm 安装"
                    )
                
                install_cmd = "bun install -g @openai/codex"
            else:  # npm
                # 检查 npm 是否可用
                if not shutil.which("npm"):
                    return InstallReport(
                        tool_name="codex",
                        result=InstallResult.FAILED,
                        message="npm 未安装，无法安装 Codex CLI",
                        error="请先安装 Node.js 和 npm"
                    )
                
                install_cmd = "npm i -g @openai/codex"
            
            # 执行安装命令
            returncode, stdout, stderr = self.run_command(
                install_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )
            
            if returncode != 0:
                return InstallReport(
                    tool_name="codex",
                    result=InstallResult.FAILED,
                    message=f"使用 {self.install_method} 安装 Codex CLI 失败",
                    error=stderr or "安装命令返回非零退出码"
                )
            
            # 验证安装
            if not self.verify():
                return InstallReport(
                    tool_name="codex",
                    result=InstallResult.FAILED,
                    message="Codex CLI 安装验证失败",
                    error="安装命令执行成功，但无法验证 Codex CLI 是否正确安装"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="codex",
                result=InstallResult.SUCCESS,
                message=f"Codex CLI 安装成功（使用 {self.install_method}）",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="codex",
                result=InstallResult.FAILED,
                message="Codex CLI 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 Codex CLI
        
        根据原安装方式执行相应的升级命令。
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="codex",
                result=InstallResult.FAILED,
                message="Codex CLI 未安装，无法升级",
                error="请先安装 Codex CLI"
            )
        
        try:
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 根据安装方式执行相应的升级命令
            if self.install_method == 'bun':
                # 检查 Bun 是否可用
                if not shutil.which("bun"):
                    return InstallReport(
                        tool_name="codex",
                        result=InstallResult.FAILED,
                        message="Bun 未安装，无法使用 Bun 升级 Codex CLI",
                        error="请先安装 Bun 或使用 npm 升级"
                    )
                
                upgrade_cmd = "bun update -g @openai/codex"
            else:  # npm
                # 检查 npm 是否可用
                if not shutil.which("npm"):
                    return InstallReport(
                        tool_name="codex",
                        result=InstallResult.FAILED,
                        message="npm 未安装，无法升级 Codex CLI",
                        error="请先安装 Node.js 和 npm"
                    )
                
                upgrade_cmd = "npm update -g @openai/codex"
            
            # 执行升级命令
            returncode, stdout, stderr = self.run_command(
                upgrade_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )
            
            if returncode != 0:
                return InstallReport(
                    tool_name="codex",
                    result=InstallResult.FAILED,
                    message=f"使用 {self.install_method} 升级 Codex CLI 失败",
                    error=stderr or "升级命令返回非零退出码"
                )
            
            # 验证升级
            if not self.verify():
                return InstallReport(
                    tool_name="codex",
                    result=InstallResult.FAILED,
                    message="Codex CLI 升级验证失败",
                    error="升级命令执行成功，但无法验证 Codex CLI 是否正确安装"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="codex",
                result=InstallResult.SUCCESS,
                message=f"Codex CLI 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="codex",
                result=InstallResult.FAILED,
                message="Codex CLI 升级过程中发生异常",
                error=str(e)
            )



"""错误处理模块单元测试"""

import pytest
from unittest.mock import MagicMock

from mono_kickstart.errors import (
    ExitCode,
    MonoKickstartError,
    PlatformNotSupportedError,
    PythonVersionError,
    ConfigError,
    ToolInstallError,
    NetworkError,
    PermissionError,
    DependencyError,
    format_error_message,
    handle_error
)


class TestExitCode:
    """退出码测试"""
    
    def test_exit_code_values(self):
        """测试退出码值"""
        assert ExitCode.SUCCESS.value == 0
        assert ExitCode.GENERAL_ERROR.value == 1
        assert ExitCode.CONFIG_ERROR.value == 2
        assert ExitCode.ALL_TASKS_FAILED.value == 3
        assert ExitCode.PERMISSION_ERROR.value == 4
        assert ExitCode.DEPENDENCY_ERROR.value == 5
        assert ExitCode.USER_INTERRUPT.value == 130


class TestMonoKickstartError:
    """基础异常类测试"""
    
    def test_init_with_message_only(self):
        """测试仅使用消息初始化"""
        error = MonoKickstartError("Test error")
        
        assert error.message == "Test error"
        assert error.exit_code == ExitCode.GENERAL_ERROR
        assert error.recovery_suggestions == []
    
    def test_init_with_all_parameters(self):
        """测试使用所有参数初始化"""
        suggestions = ["Suggestion 1", "Suggestion 2"]
        error = MonoKickstartError(
            message="Test error",
            exit_code=ExitCode.CONFIG_ERROR,
            recovery_suggestions=suggestions
        )
        
        assert error.message == "Test error"
        assert error.exit_code == ExitCode.CONFIG_ERROR
        assert error.recovery_suggestions == suggestions
    
    def test_str_representation(self):
        """测试字符串表示"""
        error = MonoKickstartError("Test error")
        
        assert str(error) == "Test error"


class TestPlatformNotSupportedError:
    """平台不支持错误测试"""
    
    def test_init(self):
        """测试初始化"""
        error = PlatformNotSupportedError(os="windows", arch="x86_64")
        
        assert "windows" in error.message
        assert "x86_64" in error.message
        assert error.os == "windows"
        assert error.arch == "x86_64"
        assert error.exit_code == ExitCode.GENERAL_ERROR
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = PlatformNotSupportedError(os="windows", arch="x86_64")
        
        assert len(error.recovery_suggestions) > 0
        assert any("macOS" in s for s in error.recovery_suggestions)
        assert any("Linux" in s for s in error.recovery_suggestions)


class TestPythonVersionError:
    """Python 版本错误测试"""
    
    def test_init_with_default_required_version(self):
        """测试使用默认要求版本初始化"""
        error = PythonVersionError(current_version="3.9.0")
        
        assert "3.9.0" in error.message
        assert "3.11" in error.message
        assert error.current_version == "3.9.0"
        assert error.required_version == "3.11"
    
    def test_init_with_custom_required_version(self):
        """测试使用自定义要求版本初始化"""
        error = PythonVersionError(
            current_version="3.9.0",
            required_version="3.12"
        )
        
        assert "3.9.0" in error.message
        assert "3.12" in error.message
        assert error.required_version == "3.12"
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = PythonVersionError(current_version="3.9.0")
        
        assert len(error.recovery_suggestions) > 0
        assert any("pyenv" in s for s in error.recovery_suggestions)
        assert any("Conda" in s for s in error.recovery_suggestions)


class TestConfigError:
    """配置错误测试"""
    
    def test_init_without_config_file(self):
        """测试不指定配置文件初始化"""
        error = ConfigError(message="Invalid format")
        
        assert "Invalid format" in error.message
        assert error.config_file is None
        assert error.exit_code == ExitCode.CONFIG_ERROR
    
    def test_init_with_config_file(self):
        """测试指定配置文件初始化"""
        error = ConfigError(
            message="Invalid format",
            config_file=".kickstartrc"
        )
        
        assert "Invalid format" in error.message
        assert ".kickstartrc" in error.message
        assert error.config_file == ".kickstartrc"
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = ConfigError(message="Invalid format")
        
        assert len(error.recovery_suggestions) > 0
        assert any("YAML" in s for s in error.recovery_suggestions)


class TestToolInstallError:
    """工具安装错误测试"""
    
    def test_init_without_manual_guide(self):
        """测试不提供手动安装指引初始化"""
        error = ToolInstallError(
            tool_name="nvm",
            reason="Network timeout"
        )
        
        assert "nvm" in error.message
        assert "Network timeout" in error.message
        assert error.tool_name == "nvm"
        assert error.reason == "Network timeout"
    
    def test_init_with_manual_guide(self):
        """测试提供手动安装指引初始化"""
        error = ToolInstallError(
            tool_name="nvm",
            reason="Network timeout",
            manual_install_guide="https://github.com/nvm-sh/nvm#install"
        )
        
        assert len(error.recovery_suggestions) > 0
        assert any("https://github.com/nvm-sh/nvm#install" in s 
                  for s in error.recovery_suggestions)
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = ToolInstallError(tool_name="nvm", reason="Network timeout")
        
        assert len(error.recovery_suggestions) > 0
        assert any("网络" in s for s in error.recovery_suggestions)


class TestNetworkError:
    """网络错误测试"""
    
    def test_init(self):
        """测试初始化"""
        error = NetworkError(
            url="https://example.com/file.tar.gz",
            reason="Connection timeout"
        )
        
        assert "https://example.com/file.tar.gz" in error.message
        assert "Connection timeout" in error.message
        assert error.url == "https://example.com/file.tar.gz"
        assert error.reason == "Connection timeout"
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = NetworkError(url="https://example.com", reason="Timeout")
        
        assert len(error.recovery_suggestions) > 0
        assert any("网络" in s for s in error.recovery_suggestions)
        assert any("镜像" in s for s in error.recovery_suggestions)


class TestPermissionError:
    """权限错误测试"""
    
    def test_init(self):
        """测试初始化"""
        error = PermissionError(
            path="/usr/local/bin/tool",
            operation="写入"
        )
        
        assert "/usr/local/bin/tool" in error.message
        assert "写入" in error.message
        assert error.path == "/usr/local/bin/tool"
        assert error.operation == "写入"
        assert error.exit_code == ExitCode.PERMISSION_ERROR
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = PermissionError(path="/usr/local/bin", operation="写入")
        
        assert len(error.recovery_suggestions) > 0
        assert any("权限" in s for s in error.recovery_suggestions)


class TestDependencyError:
    """依赖错误测试"""
    
    def test_init_without_install_command(self):
        """测试不提供安装命令初始化"""
        error = DependencyError(
            dependency="git",
            required_by="nvm"
        )
        
        assert "git" in error.message
        assert "nvm" in error.message
        assert error.dependency == "git"
        assert error.required_by == "nvm"
        assert error.exit_code == ExitCode.DEPENDENCY_ERROR
    
    def test_init_with_install_command(self):
        """测试提供安装命令初始化"""
        error = DependencyError(
            dependency="git",
            required_by="nvm",
            install_command="apt-get install git"
        )
        
        assert len(error.recovery_suggestions) > 0
        assert any("apt-get install git" in s for s in error.recovery_suggestions)
    
    def test_recovery_suggestions(self):
        """测试恢复建议"""
        error = DependencyError(dependency="git", required_by="nvm")
        
        assert len(error.recovery_suggestions) > 0


class TestFormatErrorMessage:
    """格式化错误消息测试"""
    
    def test_format_error_with_suggestions(self):
        """测试格式化带建议的错误"""
        error = MonoKickstartError(
            message="Test error",
            recovery_suggestions=["Suggestion 1", "Suggestion 2"]
        )
        
        formatted = format_error_message(error)
        
        assert "Test error" in formatted
        assert "恢复建议" in formatted
        assert "Suggestion 1" in formatted
        assert "Suggestion 2" in formatted
    
    def test_format_error_without_suggestions(self):
        """测试格式化无建议的错误"""
        error = MonoKickstartError(message="Test error")
        
        formatted = format_error_message(error)
        
        assert "Test error" in formatted
        assert "恢复建议" not in formatted


class TestHandleError:
    """错误处理测试"""
    
    def test_handle_mono_kickstart_error(self):
        """测试处理自定义错误"""
        logger = MagicMock()
        error = MonoKickstartError(
            message="Test error",
            exit_code=ExitCode.CONFIG_ERROR
        )
        
        exit_code = handle_error(error, logger)
        
        assert exit_code == ExitCode.CONFIG_ERROR.value
        logger.error.assert_called_once()
    
    def test_handle_keyboard_interrupt(self):
        """测试处理用户中断"""
        logger = MagicMock()
        error = KeyboardInterrupt()
        
        exit_code = handle_error(error, logger)
        
        assert exit_code == ExitCode.USER_INTERRUPT.value
        logger.error.assert_called_once()
        assert "中断" in logger.error.call_args[0][0]
    
    def test_handle_unknown_error(self):
        """测试处理未知错误"""
        logger = MagicMock()
        error = ValueError("Unknown error")
        
        exit_code = handle_error(error, logger)
        
        assert exit_code == ExitCode.GENERAL_ERROR.value
        logger.error.assert_called()
        logger.debug.assert_called_once()
    
    def test_handle_error_calls_logger_methods(self):
        """测试错误处理调用日志方法"""
        logger = MagicMock()
        error = MonoKickstartError(
            message="Test error",
            recovery_suggestions=["Suggestion"]
        )
        
        handle_error(error, logger)
        
        # 验证 logger.error 被调用
        logger.error.assert_called_once()
        
        # 验证错误消息包含建议
        error_message = logger.error.call_args[0][0]
        assert "Test error" in error_message
        assert "Suggestion" in error_message



"""
平台检测器单元测试

测试平台检测器的各种场景，包括：
- OS 检测（macOS、Linux、不支持）
- 架构检测（ARM64、x86_64、不支持）
- Shell 检测（Bash、Zsh、Fish、未知）
- Shell 配置文件路径获取
- 平台支持性检查
"""

import os
from pathlib import Path
from unittest.mock import patch, MagicMock

import pytest

from mono_kickstart.platform_detector import (
    OS,
    Arch,
    Shell,
    PlatformInfo,
    PlatformDetector,
)


class TestOSDetection:
    """测试操作系统检测"""

    def test_detect_macos(self):
        """测试 macOS 检测"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Darwin"):
            assert detector.detect_os() == OS.MACOS

    def test_detect_linux(self):
        """测试 Linux 检测"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Linux"):
            assert detector.detect_os() == OS.LINUX

    def test_detect_unsupported_os(self):
        """测试不支持的操作系统"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Windows"):
            assert detector.detect_os() == OS.UNSUPPORTED

    def test_detect_os_case_insensitive(self):
        """测试 OS 检测的大小写不敏感性"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="DARWIN"):
            assert detector.detect_os() == OS.MACOS
        with patch("platform.system", return_value="linux"):
            assert detector.detect_os() == OS.LINUX


class TestArchDetection:
    """测试 CPU 架构检测"""

    def test_detect_arm64(self):
        """测试 ARM64 架构检测"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="arm64"):
            assert detector.detect_arch() == Arch.ARM64

    def test_detect_aarch64(self):
        """测试 aarch64 架构检测（ARM64 的另一种表示）"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="aarch64"):
            assert detector.detect_arch() == Arch.ARM64

    def test_detect_x86_64(self):
        """测试 x86_64 架构检测"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="x86_64"):
            assert detector.detect_arch() == Arch.X86_64

    def test_detect_amd64(self):
        """测试 amd64 架构检测（x86_64 的另一种表示）"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="amd64"):
            assert detector.detect_arch() == Arch.X86_64

    def test_detect_unsupported_arch(self):
        """测试不支持的架构"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="i386"):
            assert detector.detect_arch() == Arch.UNSUPPORTED

    def test_detect_arch_case_insensitive(self):
        """测试架构检测的大小写不敏感性"""
        detector = PlatformDetector()
        with patch("platform.machine", return_value="ARM64"):
            assert detector.detect_arch() == Arch.ARM64
        with patch("platform.machine", return_value="X86_64"):
            assert detector.detect_arch() == Arch.X86_64


class TestShellDetection:
    """测试 Shell 类型检测"""

    def test_detect_bash(self):
        """测试 Bash 检测"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {"SHELL": "/bin/bash"}):
            assert detector.detect_shell() == Shell.BASH

    def test_detect_zsh(self):
        """测试 Zsh 检测"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {"SHELL": "/bin/zsh"}):
            assert detector.detect_shell() == Shell.ZSH

    def test_detect_fish(self):
        """测试 Fish 检测"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {"SHELL": "/usr/bin/fish"}):
            assert detector.detect_shell() == Shell.FISH

    def test_detect_unknown_shell(self):
        """测试未知 Shell"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {"SHELL": "/bin/ksh"}):
            assert detector.detect_shell() == Shell.UNKNOWN

    def test_detect_shell_no_env_var(self):
        """测试 SHELL 环境变量不存在的情况"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {}, clear=True):
            assert detector.detect_shell() == Shell.UNKNOWN

    def test_detect_shell_case_insensitive(self):
        """测试 Shell 检测的大小写不敏感性"""
        detector = PlatformDetector()
        with patch.dict(os.environ, {"SHELL": "/bin/BASH"}):
            assert detector.detect_shell() == Shell.BASH


class TestShellConfigFile:
    """测试 Shell 配置文件路径获取"""

    def test_bash_config_file_bashrc_exists(self, tmp_path):
        """测试 Bash 配置文件（.bashrc 存在）"""
        detector = PlatformDetector()
        bashrc = tmp_path / ".bashrc"
        bashrc.touch()
        
        with patch("pathlib.Path.home", return_value=tmp_path):
            config_file = detector.get_shell_config_file(Shell.BASH)
            assert config_file == str(bashrc)

    def test_bash_config_file_bashrc_not_exists(self, tmp_path):
        """测试 Bash 配置文件（.bashrc 不存在，使用 .bash_profile）"""
        detector = PlatformDetector()
        bash_profile = tmp_path / ".bash_profile"
        
        with patch("pathlib.Path.home", return_value=tmp_path):
            config_file = detector.get_shell_config_file(Shell.BASH)
            assert config_file == str(bash_profile)

    def test_zsh_config_file(self, tmp_path):
        """测试 Zsh 配置文件"""
        detector = PlatformDetector()
        zshrc = tmp_path / ".zshrc"
        
        with patch("pathlib.Path.home", return_value=tmp_path):
            config_file = detector.get_shell_config_file(Shell.ZSH)
            assert config_file == str(zshrc)

    def test_fish_config_file(self, tmp_path):
        """测试 Fish 配置文件"""
        detector = PlatformDetector()
        fish_config = tmp_path / ".config" / "fish" / "config.fish"
        
        with patch("pathlib.Path.home", return_value=tmp_path):
            config_file = detector.get_shell_config_file(Shell.FISH)
            assert config_file == str(fish_config)

    def test_unknown_shell_config_file(self, tmp_path):
        """测试未知 Shell 的配置文件（使用 .profile 作为后备）"""
        detector = PlatformDetector()
        profile = tmp_path / ".profile"
        
        with patch("pathlib.Path.home", return_value=tmp_path):
            config_file = detector.get_shell_config_file(Shell.UNKNOWN)
            assert config_file == str(profile)


class TestDetectAll:
    """测试完整平台信息检测"""

    def test_detect_all_macos_arm64_zsh(self, tmp_path):
        """测试 macOS ARM64 + Zsh 的完整检测"""
        detector = PlatformDetector()
        zshrc = tmp_path / ".zshrc"
        
        with patch("platform.system", return_value="Darwin"), \
             patch("platform.machine", return_value="arm64"), \
             patch.dict(os.environ, {"SHELL": "/bin/zsh"}), \
             patch("pathlib.Path.home", return_value=tmp_path):
            
            info = detector.detect_all()
            
            assert info.os == OS.MACOS
            assert info.arch == Arch.ARM64
            assert info.shell == Shell.ZSH
            assert info.shell_config_file == str(zshrc)

    def test_detect_all_linux_x86_64_bash(self, tmp_path):
        """测试 Linux x86_64 + Bash 的完整检测"""
        detector = PlatformDetector()
        bashrc = tmp_path / ".bashrc"
        bashrc.touch()
        
        with patch("platform.system", return_value="Linux"), \
             patch("platform.machine", return_value="x86_64"), \
             patch.dict(os.environ, {"SHELL": "/bin/bash"}), \
             patch("pathlib.Path.home", return_value=tmp_path):
            
            info = detector.detect_all()
            
            assert info.os == OS.LINUX
            assert info.arch == Arch.X86_64
            assert info.shell == Shell.BASH
            assert info.shell_config_file == str(bashrc)

    def test_detect_all_unsupported_platform(self, tmp_path):
        """测试不支持的平台的完整检测"""
        detector = PlatformDetector()
        
        with patch("platform.system", return_value="Windows"), \
             patch("platform.machine", return_value="x86_64"), \
             patch.dict(os.environ, {"SHELL": "/bin/bash"}), \
             patch("pathlib.Path.home", return_value=tmp_path):
            
            info = detector.detect_all()
            
            assert info.os == OS.UNSUPPORTED
            assert info.arch == Arch.X86_64
            assert info.shell == Shell.BASH


class TestPlatformSupport:
    """测试平台支持性检查"""

    def test_macos_arm64_supported(self):
        """测试 macOS ARM64 是支持的平台"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Darwin"), \
             patch("platform.machine", return_value="arm64"):
            assert detector.is_supported() is True

    def test_macos_x86_64_supported(self):
        """测试 macOS x86_64 是支持的平台"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Darwin"), \
             patch("platform.machine", return_value="x86_64"):
            assert detector.is_supported() is True

    def test_linux_x86_64_supported(self):
        """测试 Linux x86_64 是支持的平台"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Linux"), \
             patch("platform.machine", return_value="x86_64"):
            assert detector.is_supported() is True

    def test_linux_arm64_not_supported(self):
        """测试 Linux ARM64 不是支持的平台"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Linux"), \
             patch("platform.machine", return_value="arm64"):
            assert detector.is_supported() is False

    def test_windows_not_supported(self):
        """测试 Windows 不是支持的平台"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Windows"), \
             patch("platform.machine", return_value="x86_64"):
            assert detector.is_supported() is False

    def test_unsupported_arch_not_supported(self):
        """测试不支持的架构"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="Darwin"), \
             patch("platform.machine", return_value="i386"):
            assert detector.is_supported() is False

    def test_unsupported_os_not_supported(self):
        """测试不支持的操作系统"""
        detector = PlatformDetector()
        with patch("platform.system", return_value="FreeBSD"), \
             patch("platform.machine", return_value="x86_64"):
            assert detector.is_supported() is False


class TestPlatformInfoDataclass:
    """测试 PlatformInfo 数据类"""

    def test_platform_info_creation(self):
        """测试 PlatformInfo 对象创建"""
        info = PlatformInfo(
            os=OS.MACOS,
            arch=Arch.ARM64,
            shell=Shell.ZSH,
            shell_config_file="/Users/test/.zshrc",
        )
        
        assert info.os == OS.MACOS
        assert info.arch == Arch.ARM64
        assert info.shell == Shell.ZSH
        assert info.shell_config_file == "/Users/test/.zshrc"

    def test_platform_info_equality(self):
        """测试 PlatformInfo 对象相等性"""
        info1 = PlatformInfo(
            os=OS.MACOS,
            arch=Arch.ARM64,
            shell=Shell.ZSH,
            shell_config_file="/Users/test/.zshrc",
        )
        info2 = PlatformInfo(
            os=OS.MACOS,
            arch=Arch.ARM64,
            shell=Shell.ZSH,
            shell_config_file="/Users/test/.zshrc",
        )
        
        assert info1 == info2

    def test_platform_info_inequality(self):
        """测试 PlatformInfo 对象不相等性"""
        info1 = PlatformInfo(
            os=OS.MACOS,
            arch=Arch.ARM64,
            shell=Shell.ZSH,
            shell_config_file="/Users/test/.zshrc",
        )
        info2 = PlatformInfo(
            os=OS.LINUX,
            arch=Arch.X86_64,
            shell=Shell.BASH,
            shell_config_file="/home/test/.bashrc",
        )
        
        assert info1 != info2



"""工具安装器基类模块

该模块定义了工具安装器的通用接口和基础功能，包括命令执行、文件下载等。
所有具体的工具安装器都应继承自 ToolInstaller 抽象基类。
"""

import subprocess
import time
import urllib.request
import urllib.error
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Optional, Tuple

from .config import ToolConfig
from .platform_detector import PlatformInfo


class InstallResult(Enum):
    """安装结果枚举
    
    Attributes:
        SUCCESS: 安装成功
        SKIPPED: 跳过安装（工具已存在）
        FAILED: 安装失败
    """
    SUCCESS = "success"
    SKIPPED = "skipped"
    FAILED = "failed"


@dataclass
class InstallReport:
    """安装报告
    
    Attributes:
        tool_name: 工具名称
        result: 安装结果
        message: 描述信息
        version: 安装的版本号（可选）
        error: 错误信息（可选）
    """
    tool_name: str
    result: InstallResult
    message: str
    version: Optional[str] = None
    error: Optional[str] = None


class ToolInstaller(ABC):
    """工具安装器抽象基类
    
    所有具体的工具安装器都应继承此类并实现抽象方法。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化工具安装器
        
        Args:
            platform_info: 平台信息（操作系统、架构、Shell 等）
            config: 工具配置
        """
        self.platform_info = platform_info
        self.config = config
    
    @abstractmethod
    def install(self) -> InstallReport:
        """安装工具
        
        子类必须实现此方法以提供具体的安装逻辑。
        
        Returns:
            InstallReport: 安装报告
        """
        pass
    
    @abstractmethod
    def upgrade(self) -> InstallReport:
        """升级工具
        
        子类必须实现此方法以提供具体的升级逻辑。
        
        Returns:
            InstallReport: 升级报告
        """
        pass
    
    @abstractmethod
    def verify(self) -> bool:
        """验证安装是否成功
        
        子类必须实现此方法以验证工具是否正确安装。
        
        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        pass
    
    def run_command(
        self,
        command: str,
        shell: bool = True,
        timeout: int = 300,
        max_retries: int = 3,
        retry_delay: int = 1
    ) -> Tuple[int, str, str]:
        """执行 shell 命令（支持重试）
        
        Args:
            command: 要执行的命令
            shell: 是否使用 shell 执行（默认为 True）
            timeout: 命令超时时间（秒），默认 300 秒
            max_retries: 最大重试次数，默认 3 次
            retry_delay: 重试延迟（秒），使用指数退避，默认初始延迟 1 秒
            
        Returns:
            Tuple[int, str, str]: (返回码, stdout, stderr)
            
        Raises:
            subprocess.TimeoutExpired: 命令执行超时
        """
        last_exception = None
        current_delay = retry_delay
        
        for attempt in range(max_retries):
            try:
                result = subprocess.run(
                    command,
                    shell=shell,
                    capture_output=True,
                    text=True,
                    timeout=timeout
                )
                
                return (result.returncode, result.stdout, result.stderr)
                
            except subprocess.TimeoutExpired as e:
                # 超时错误不重试，直接抛出
                raise
                
            except subprocess.SubprocessError as e:
                last_exception = e
                
                # 如果不是最后一次尝试，等待后重试
                if attempt < max_retries - 1:
                    time.sleep(current_delay)
                    # 指数退避：每次重试延迟翻倍
                    current_delay *= 2
        
        # 所有重试都失败，返回错误信息
        error_msg = f"命令执行失败: {str(last_exception)}"
        return (-1, "", error_msg)
    
    def download_file(
        self,
        url: str,
        dest: str,
        max_retries: int = 3,
        retry_delay: int = 1,
        timeout: int = 300
    ) -> bool:
        """下载文件（支持重试）
        
        Args:
            url: 文件 URL
            dest: 目标文件路径
            max_retries: 最大重试次数，默认 3 次
            retry_delay: 重试延迟（秒），使用指数退避，默认初始延迟 1 秒
            timeout: 下载超时时间（秒），默认 300 秒
            
        Returns:
            bool: 如果下载成功返回 True，否则返回 False
        """
        dest_path = Path(dest)
        
        # 确保目标目录存在
        dest_path.parent.mkdir(parents=True, exist_ok=True)
        
        last_exception = None
        current_delay = retry_delay
        
        for attempt in range(max_retries):
            try:
                # 下载文件
                with urllib.request.urlopen(url, timeout=timeout) as response:
                    with open(dest_path, 'wb') as out_file:
                        out_file.write(response.read())
                
                # 验证文件是否存在且非空
                if dest_path.exists() and dest_path.stat().st_size > 0:
                    return True
                    
            except (urllib.error.URLError, urllib.error.HTTPError, OSError) as e:
                last_exception = e
                
                # 如果文件部分下载，删除它
                if dest_path.exists():
                    try:
                        dest_path.unlink()
                    except OSError:
                        pass
                
                # 如果不是最后一次尝试，等待后重试
                if attempt < max_retries - 1:
                    time.sleep(current_delay)
                    # 指数退避：每次重试延迟翻倍
                    current_delay *= 2
        
        # 所有重试都失败
        return False



"""NVM 安装器模块

该模块实现 NVM (Node Version Manager) 的安装、升级和验证逻辑。
"""

import os
import tempfile
from pathlib import Path
from typing import Optional

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo, Shell


class NVMInstaller(ToolInstaller):
    """NVM 安装器
    
    负责安装、升级和验证 NVM (Node Version Manager)。
    NVM 通过下载并执行官方安装脚本进行安装，并将环境变量写入 Shell 配置文件。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        nvm_version: NVM 版本（默认 v0.40.4）
        install_script_url: 安装脚本 URL
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 NVM 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # 从配置中获取版本，如果未指定则使用默认版本
        self.nvm_version = config.version or "v0.40.4"
        
        # 构建安装脚本 URL
        self.install_script_url = (
            f"https://raw.githubusercontent.com/nvm-sh/nvm/{self.nvm_version}/install.sh"
        )
    
    def verify(self) -> bool:
        """验证 NVM 是否已正确安装
        
        检查 ~/.nvm/nvm.sh 文件是否存在，并尝试执行 nvm --version 命令。
        
        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        nvm_dir = Path.home() / ".nvm"
        nvm_sh = nvm_dir / "nvm.sh"
        
        # 检查 nvm.sh 文件是否存在
        if not nvm_sh.exists():
            return False
        
        # 尝试执行 nvm --version 命令
        returncode, stdout, stderr = self.run_command(
            f"bash -c 'source {nvm_sh} && nvm --version'",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 NVM 版本
        
        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        nvm_sh = Path.home() / ".nvm" / "nvm.sh"
        
        if not nvm_sh.exists():
            return None
        
        returncode, stdout, stderr = self.run_command(
            f"bash -c 'source {nvm_sh} && nvm --version'",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        if returncode == 0:
            return stdout.strip()
        
        return None
    
    def _write_env_to_shell_config(self) -> bool:
        """将 NVM 环境变量写入 Shell 配置文件
        
        根据当前 Shell 类型，将 NVM 的环境变量配置写入相应的配置文件。
        如果配置已存在，则不重复写入。
        
        Returns:
            bool: 如果写入成功返回 True，否则返回 False
        """
        shell_config_file = Path(self.platform_info.shell_config_file)
        
        # NVM 环境变量配置
        nvm_config_lines = [
            '# NVM configuration',
            'export NVM_DIR="$HOME/.nvm"',
            '[ -s "$NVM_DIR/nvm.sh" ] && \\. "$NVM_DIR/nvm.sh"  # This loads nvm',
            '[ -s "$NVM_DIR/bash_completion" ] && \\. "$NVM_DIR/bash_completion"  # This loads nvm bash_completion',
        ]
        
        try:
            # 读取现有配置文件内容
            if shell_config_file.exists():
                with open(shell_config_file, 'r', encoding='utf-8') as f:
                    existing_content = f.read()
            else:
                existing_content = ""
            
            # 检查是否已经包含 NVM 配置
            if 'NVM_DIR' in existing_content and 'nvm.sh' in existing_content:
                # 配置已存在，无需重复写入
                return True
            
            # 追加 NVM 配置到文件末尾
            with open(shell_config_file, 'a', encoding='utf-8') as f:
                # 确保文件末尾有换行符
                if existing_content and not existing_content.endswith('\n'):
                    f.write('\n')
                
                # 写入配置
                f.write('\n')
                for line in nvm_config_lines:
                    f.write(line + '\n')
            
            return True
            
        except (OSError, IOError) as e:
            # 文件操作失败
            return False
    
    def install(self) -> InstallReport:
        """安装 NVM
        
        执行以下步骤：
        1. 检查 NVM 是否已安装
        2. 下载 NVM 安装脚本
        3. 执行安装脚本
        4. 将环境变量写入 Shell 配置文件
        5. 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.SKIPPED,
                message="NVM 已安装，跳过安装",
                version=version
            )
        
        try:
            # 下载安装脚本到临时文件
            with tempfile.NamedTemporaryFile(
                mode='w',
                suffix='.sh',
                delete=False
            ) as temp_file:
                temp_script_path = temp_file.name
            
            # 下载安装脚本
            download_success = self.download_file(
                self.install_script_url,
                temp_script_path,
                max_retries=3,
                timeout=60
            )
            
            if not download_success:
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="下载 NVM 安装脚本失败",
                    error="无法从 GitHub 下载安装脚本"
                )
            
            # 执行安装脚本
            returncode, stdout, stderr = self.run_command(
                f"bash {temp_script_path}",
                shell=True,
                timeout=300,
                max_retries=1
            )
            
            # 清理临时文件
            try:
                Path(temp_script_path).unlink()
            except OSError:
                pass
            
            if returncode != 0:
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="执行 NVM 安装脚本失败",
                    error=stderr or "安装脚本返回非零退出码"
                )
            
            # 将环境变量写入 Shell 配置文件
            env_write_success = self._write_env_to_shell_config()
            
            if not env_write_success:
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="写入 Shell 配置文件失败",
                    error="无法将 NVM 环境变量写入 Shell 配置文件"
                )
            
            # 验证安装
            if not self.verify():
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="NVM 安装验证失败",
                    error="安装脚本执行成功，但无法验证 NVM 是否正确安装"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.SUCCESS,
                message="NVM 安装成功",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.FAILED,
                message="NVM 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 NVM
        
        NVM 的升级通过重新执行安装脚本实现。
        安装脚本会自动检测已有安装并进行升级。
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.FAILED,
                message="NVM 未安装，无法升级",
                error="请先安装 NVM"
            )
        
        try:
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 下载安装脚本到临时文件
            with tempfile.NamedTemporaryFile(
                mode='w',
                suffix='.sh',
                delete=False
            ) as temp_file:
                temp_script_path = temp_file.name
            
            # 下载安装脚本
            download_success = self.download_file(
                self.install_script_url,
                temp_script_path,
                max_retries=3,
                timeout=60
            )
            
            if not download_success:
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="下载 NVM 安装脚本失败",
                    error="无法从 GitHub 下载安装脚本"
                )
            
            # 执行安装脚本（会自动升级）
            returncode, stdout, stderr = self.run_command(
                f"bash {temp_script_path}",
                shell=True,
                timeout=300,
                max_retries=1
            )
            
            # 清理临时文件
            try:
                Path(temp_script_path).unlink()
            except OSError:
                pass
            
            if returncode != 0:
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="执行 NVM 升级脚本失败",
                    error=stderr or "升级脚本返回非零退出码"
                )
            
            # 验证升级
            if not self.verify():
                return InstallReport(
                    tool_name="nvm",
                    result=InstallResult.FAILED,
                    message="NVM 升级验证失败",
                    error="升级脚本执行成功，但无法验证 NVM 是否正确安装"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.SUCCESS,
                message=f"NVM 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="nvm",
                result=InstallResult.FAILED,
                message="NVM 升级过程中发生异常",
                error=str(e)
            )
